// Process.cpp : Implementation of Data Processing Menu Option
//

#include "stdafx.h"
#include "FEMvrml.h"
#include "MainFrm.h"

#include "Process.h"
#include "FEMvrmlDoc.h"
#include "FEMvrmlView.h"

#include <string>
using namespace std;
void makeDAT(string& inputDynNAME, string& inputTmeNAME, string& outputNAME,
			 double criteriaValue, int sColorOption);
void makeWRL(string& inputTmeNAME, string& outputNAME, int aOption, 
			 double vSCALE_D, double vSCALE_N, int x3dOption);

// CProcess dialog box

IMPLEMENT_DYNAMIC(CProcess, CDialog)
CProcess::CProcess(CWnd* pParent /*=NULL*/)
	: CDialog(CProcess::IDD, pParent)
	, m_input_dyn_file_name(_T(""))
	, m_input_tme_file_name(_T(""))
	, m_output_file_name(_T(""))
	, m_iscale_d(0)
	, m_vscale_d(0)
	, m_iscale_n(0)
	, m_vscale_n(0)
	, m_aoption(0)
	, m_display_file_name(_T(""))
	, m_criteria_deletion(0)
{
}

// m_input_dyn_file_name : input *.DYN filename w/ extension 
// m_input_tme_file_name : input displacement-history filename w/extension
// m_output_file_name : output filename (*.DAT, *.WRL/*.X3D) w/o extension
// m_iscale_d : changed to x3dOption : printout option for VRML = 1, X3D = 2
// m_vscale_d : scale factor for deformed shape
// m_iscale_n : changed to sColorOption 1: same color 2: different color for deleted part with original part
// m_vscale_n : relative object size in browser
// m_aoption : 1 static VRML (Geometry Only) & 2 dynamic static VRML (Simulation)
// m_display_file_name : display filename => m_output_file_name + ".htm"
// m_criteria_deletion : criteria value for element deletion (elongation)

CProcess::~CProcess()
{
}

void CProcess::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_EDIT1, m_input_dyn_file_name);
	DDX_Text(pDX, IDC_EDIT2, m_input_tme_file_name);
	DDX_Text(pDX, IDC_EDIT3, m_output_file_name);
	DDX_Text(pDX, IDC_EDIT4, m_iscale_d);
	DDX_Text(pDX, IDC_EDIT7, m_vscale_d);
	DDX_Text(pDX, IDC_EDIT5, m_iscale_n);
	DDX_Text(pDX, IDC_EDIT6, m_vscale_n);
	DDX_Text(pDX, IDC_EDIT8, m_aoption);
	DDX_Text(pDX, IDC_EDIT9, m_display_file_name);
	DDX_Text(pDX, IDC_EDIT10, m_criteria_deletion);
}


BEGIN_MESSAGE_MAP(CProcess, CDialog)
	ON_COMMAND(ID_DATAPROCESSING_START, OnDataprocessingStart)
	ON_BN_CLICKED(IDC_BUTTON1, OnBnClickedButton1)
	ON_BN_CLICKED(IDC_BUTTON2, OnBnClickedButton2)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
	ON_BN_CLICKED(IDOK2, OnBnClickedOk2)
	ON_BN_CLICKED(IDCANCEL2, OnBnClickedCancel2)
	ON_BN_CLICKED(IDOK3, OnBnClickedOk3)
	ON_BN_CLICKED(IDCANCEL3, OnBnClickedCancel3)
END_MESSAGE_MAP()


// CProcess message handler

void CProcess::OnDataprocessingStart()
{
	// TODO: add commend handlers code here,
	CProcess optionDig;
	optionDig.DoModal();
}

void CProcess::OnBnClickedButton1()
{
	// TODO: add control notice handlers code here.
	CFileDialog dlg(TRUE, NULL, NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "LS-DYNA Input Files (*.dyn)|*.dyn;*.TIF|ALL Files(*.*)|*.*||" );
	if (dlg.DoModal () == IDOK) 
	{
		m_input_dyn_file_name = dlg.GetFileName();
		m_input_dyn_file_path = dlg.GetPathName();
	}
	UpdateData(FALSE);

	// make default m_output_file_name from m_input_dyn_file_name
	int num_name = m_input_dyn_file_name.GetLength();
	int num_path = m_input_dyn_file_path.GetLength();
	for (int i = 0; i < (num_path-num_name); i++)
		m_display_file_path = m_display_file_path + m_input_dyn_file_path[i];
	for (int i = (num_path-num_name); i < (num_path-4); i++)
        m_output_file_name = m_output_file_name + m_input_dyn_file_path[i];
	UpdateData(FALSE);
}

void CProcess::OnBnClickedButton2()
{
	// TODO: add control notice handlers code here.
	CFileDialog dlg(TRUE, NULL, NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "LS-DYNA Diplacement History Files (*.tme)|*.tme;*.TIF|ALL Files(*.*)|*.*||" );
	if (dlg.DoModal () == IDOK) 
	{
		m_input_tme_file_name = dlg.GetFileName();
		m_input_tme_file_path = dlg.GetPathName();
	}
	UpdateData(FALSE);

	// set default m_criteria_deletion, m_iscale_n
	m_criteria_deletion = 3.0;
	m_iscale_n = 1;
	UpdateData(FALSE);
}

void CProcess::OnBnClickedOk()
{
	// TODO: add control notice handlers code here.
	UpdateData();

	// execute conversion <step 0> process
	string inputDynNAME = m_input_dyn_file_name;
	string inputTmeNAME = m_input_tme_file_name;
	string outputNAME = m_output_file_name;
	makeDAT(inputDynNAME, inputTmeNAME, outputNAME, m_criteria_deletion, m_iscale_n);

	// set default value
	m_iscale_d = 1;
	m_aoption = 2;
	if (m_input_tme_file_name.GetLength() == 0) m_aoption = 1;
	m_vscale_d = 1.0;
	m_vscale_n = 5.0;
	UpdateData(FALSE);
}

void CProcess::OnBnClickedCancel()
{
	// TODO: add control notice handlers code here.
	OnCancel();
}

void CProcess::OnBnClickedOk2()
{
	// TODO: add control notice handlers code here.
	UpdateData();

	// execute conversion <step 1> process
	string inputTmeNAME = m_input_tme_file_name;
	string outputNAME = m_output_file_name;
	makeWRL(inputTmeNAME, outputNAME, m_aoption, m_vscale_d, m_vscale_n, m_iscale_d);

	// make default m_display_file_name from m_output_file_name
	m_display_file_name = m_output_file_name + ".htm";
	UpdateData(FALSE);

}

void CProcess::OnBnClickedCancel2()
{
	// TODO: add control notice handlers code here.
	OnCancel();
}

void CProcess::OnBnClickedOk3()
{
	// TODO: add control notice handlers code here.
	UpdateData();

	// execute display <step 2> process
	m_display_file_path = m_display_file_path + m_display_file_name;

	// open browser inside frame
	CMainFrame *pFrame = (CMainFrame *)AfxGetMainWnd(); 
	CFEMvrmlView *pView = (CFEMvrmlView *)pFrame->GetActiveView(); 
	pView->Navigate2(_T(m_display_file_path), NULL, NULL);

	OnOK();
}

void CProcess::OnBnClickedCancel3()
{
	// TODO: add control notice handlers code here.
	OnCancel();
}
