// PartObject.cpp :  implementation of class PartObject
//

#include "stdafx.h"

#include "PartObject.h"

using namespace std;

typedef int* IntArrayPtr;
typedef double* DoubleArrayPtr;
typedef string* StringArrayPtr;
typedef class PartObject* PartArrayPtr;
typedef class SectionObject* SectionArrayPtr;

PartObject::PartObject() :number_SF(1000), number_ND(1000)
{
	part_SF = new IntArrayPtr[number_SF];
	for (int i = 0; i < number_SF; i++) part_SF[i] = new int[4];
	part_ND = new int[number_ND];
}

PartObject::PartObject(int size_SF, int size_ND) :number_SF(size_SF), number_ND(size_ND)
{
	part_SF = new IntArrayPtr[number_SF];
	for (int i = 0; i < number_SF; i++) part_SF[i] = new int[4];
	part_ND = new int[number_ND];	
}

PartObject::~PartObject()
{
	for (int i = 0; i < number_SF; i++)
		delete[] part_SF[i];
	delete[] part_SF;

	delete[] part_ND;
}

// PartObject::setkPAT set kPAT,kTYPE,kpartNAME
void PartObject::setkPAT(int i, int iTYPE, string& partNAME)
{
	kPAT = i;
	kTYPE = iTYPE;
	kpartNAME = partNAME;
	kindSECTION = 0;
}

// PartObject::setARRAY set part_SF,part_ND
void PartObject::setARRAY(IntArrayPtr tempNODE, IntArrayPtr tempSURFACE)
{
	// i_number_ND : node index for this part object starting from 0 to number_ND
	// repeated : flag to check the repeated node index for storing part_ND array
	int i_number_ND = 0;
	int repeated;
	for (int i = 0; i < number_SF; i++)
	{
		for (int j = 0; j <4; j++)
		{
			repeated = 0;
			// start store node-connectivity and check repeated node
			for (int k = 0; k < i_number_ND; k++)
			{
				if(tempSURFACE[4*i+j] == tempNODE[k]) 
				{
					part_SF[i][j] = k;
					repeated = 1;
					break;
				}
			}
			// store node connectivity(Not repeated) and node index based on newly started node index
			if (repeated !=1)
			{
				part_SF[i][j] = i_number_ND;
				part_ND[i_number_ND] = tempNODE[i_number_ND];
				i_number_ND++;
			}
		}

	}
}

// PartObject::setCOLOR set COLOR automatically based on serial combination with RGB
void PartObject::setCOLOR(int i, double& xRED, double& xGREEN, double& xBLUE)
{
	double rgb_K1[11] = { 1.0, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0.0 };
	double rgb_K2[11] = { 1.0, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0.0 };
	double rgb_K3[11] = { 1.0, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0.0 };

	int order_ONE;
	int order_TWO;
	int order_THREE;

	order_ONE = i % 11;
	order_TWO =  ((i - order_ONE) / 11) % 11;
	order_THREE = (((i - order_ONE) / 11) - order_TWO) / 11;

	xRED = rgb_K1[order_ONE];
	xGREEN = rgb_K2[order_TWO];
	xBLUE = rgb_K3[order_THREE];
}

// PartObject::printVRML printout PartObject-case in *.WRL with x3dOption = 0
void PartObject::printVRML(ofstream& outVRML, DoubleArrayPtr* coordinate, DoubleArrayPtr animationTIME, 
						   IntArrayPtr inversenodeID, int caseNUMBER, int NTIM, double vSCALE_N)
{
	// color_test = 1 : color setting at material node 
	// color_test = 2 : color setting at color node
	int color_test = 1;
	// surface_type_test = 1 : quadrilateral element default with conditional triangular element
	// surface_type_test = 2 : all triangular element
	int surface_type_test = 1;
	// fraction_type_test = 1 => consider keyfraction as fraction of step number = i/NTIM
	// fraction_type_test = 2 => consider keyfraction as fractoin of each animation time = animationTIME[i] / animationTIME[NTIM]
	int fraction_type_test = 2;

	if (caseNUMBER == 1) 
	{
		// #BodySwitch Node #
		outVRML << "DEF " << kpartNAME << "_BodySwitch Switch {" << endl; // Switch START
		outVRML << "	whichChoice 0" << endl;
		outVRML << "choice [" << endl << endl;  // choice START

		// #Collision Node #
		outVRML << "DEF " << kpartNAME << "_Detect Collision {" << endl;
		outVRML << "  collide TRUE" << endl;
		outVRML << "	children [" << endl;
		outVRML << "		DEF " << kpartNAME << "_BodyTouch TouchSensor{}" << endl;

		// # Shape Node #
		outVRML << "		Shape {" << endl;

	} // end of caseNUMBER == 1

	else if (caseNUMBER == 2) 
	{
		double xRED = 0.0, xGREEN = 0.0, xBLUE = 0.0;
		setCOLOR(kPAT, xRED, xGREEN, xBLUE);

		//# appearence Node #
		if (color_test == 1) // color setting at material node
		{
			outVRML << "			appearance Appearance {" << endl;
			outVRML << "				material Material {" << endl;
			if ((kTYPE == 1) || (kTYPE == 2)) // solid or shell element - face segment
			{
				outVRML << "					diffuseColor " << xRED << " " << xGREEN << " " << xBLUE << endl;
				outVRML << "					specularColor  0.5 0.5 0.5" << endl;
				outVRML << "					shininess 0.2" << endl;
				outVRML << "					emissiveColor 0 0 0" << endl;
				outVRML << "					transparency 0" << endl;
			}
			else if (kTYPE == 3)  //beam element - line segment
			{
				outVRML << "					emissiveColor " << xRED << " " << xGREEN << " " << xBLUE << endl;
			}
			outVRML << "				}" << endl;
			outVRML << "			}" << endl;
		}

	}  // end of caseNUMBER == 2

	else if (caseNUMBER == 3) 
	{
		//# geometry Node #
		if (kTYPE == 3) // beam element - line segment
		{
			outVRML << "			geometry IndexedLineSet {" << endl;
		}
		else if (kTYPE == 2) // shell element - face segment with "solid FALSE"
		{
			outVRML << "			geometry IndexedFaceSet {" << endl;
			outVRML << "				solid FALSE" << endl;
			outVRML << "				creaseAngle 0.5" << endl;
		}
		else if (kTYPE == 1) // solid element - face segment with "solid TRUE or FALSE"
		{
			outVRML << "			geometry IndexedFaceSet {" << endl;
			outVRML << "				solid FALSE" << endl;
			outVRML << "				creaseAngle 0.5" << endl;
		}

		//# color Node #
		if (color_test == 2)  //  color setting at color node
		{
			double xRED = 0.0, xGREEN = 0.0, xBLUE = 0.0;
			setCOLOR(kPAT, xRED, xGREEN, xBLUE);

			outVRML << "				colorPerVertex TRUE" << endl;
			outVRML << "				color Color {" << endl;
			outVRML << "					color [" << endl;
			for (int i = 0; i < number_ND; i++)
			{
				outVRML << "						" << xRED << " " << xGREEN << " " << xBLUE;
				if (i <= (number_ND-2)) outVRML << "," << endl;
				if (i == (number_ND-1)) outVRML << endl;
			}
			outVRML << "					]" << endl;
			outVRML << "				}" << endl;
		}

		//# coordinate Node #
		outVRML << "				coord DEF " << kpartNAME << "_Coordinate Coordinate { " << endl;
		outVRML << "					point [" << endl;
		for (int i = 0; i < number_ND; i++)
		{
			outVRML << "						";
			for (int j = 0; j < 3 ; j++)
				outVRML << setw(20) << coordinate[inversenodeID[part_ND[i]-1]][j] * vSCALE_N;
			if (i <= (number_ND-2)) outVRML << "," << endl;
			if (i == (number_ND-1)) outVRML << endl;
		}
		outVRML << "					]" << endl;
		outVRML << "				}" << endl; // # coordinate

	//# coordIndex Node #
		outVRML << "				coordIndex [" << endl;

		for (int i = 0; i < number_SF; i++) 
		{
		    outVRML << "						";
		   if (part_SF[i][1] == part_SF[i][2]) // from beam element
			{
				for (int j = 0; j < 2; j++)
					outVRML << part_SF[i][j] << ", ";
			}
			else if (part_SF[i][0] == part_SF[i][1] ) // from wedge solid element
			{
				for (int j = 1; j < 4; j++)
					outVRML << part_SF[i][j] << ", ";
			}
			else if  (part_SF[i][2] == part_SF[i][3] ) // from triangle shell/plate element
			{
				for (int j = 0; j < 3; j++)
					outVRML << part_SF[i][j] << ", ";
			}
			else // from rectangle shell/plate element or rectangle faec of solid element
			{
				if (surface_type_test == 1) // quadrilateral element default
				{
					for (int j = 0; j < 4; j++)
						outVRML << part_SF[i][j] << ", ";
				}
				else if (surface_type_test == 2) // all triangular element
				{
					outVRML << part_SF[i][0] << ", " << part_SF[i][1] << ", " << part_SF[i][2] << ", -1,  ";
					outVRML << part_SF[i][2] << ", " << part_SF[i][3] << ", " << part_SF[i][0] << ", ";
				}
			}
			if (i <= (number_SF-2)) outVRML << "-1," << endl;
			if (i == (number_SF-1)) outVRML << "-1" << endl;
		}
		outVRML << "				]" << endl; // #coordIndex

		outVRML << "			}" << endl; // #geometry
		outVRML << "		}" << endl << endl;  // #Shape

	} // end of caseNUMBER ==3

	else if (caseNUMBER == 4)
	{
		double keyfraction;

		//# CoordinateInterpolator Node #
		outVRML << "		DEF " << kpartNAME << "_Morph CoordinateInterpolator {" << endl;

		//# key Node #
		outVRML << "			key [";
		for (int i = 0 ; i < NTIM+1 ; i++)
		{
			if (fraction_type_test == 1) keyfraction = double(i) / double (NTIM); // fraction based on step number
			else if (fraction_type_test == 2) keyfraction = double (animationTIME[i] / animationTIME[NTIM]); // fraction based on animation time
			outVRML << keyfraction << " ";
			if ((i != 0) && ((i%10) == 0)) outVRML << endl << "			     ";
		}
		outVRML << "]" << endl; //# key

		//# KeyValue Node #
		outVRML << "			keyValue [" << endl;

	} // end of caseNUMBER == 4


	else if (caseNUMBER == 5)
	{
		outVRML << endl;
		for (int i = 0; i < number_ND; i++)
		{
			outVRML << "						";
			for (int j = 0; j < 3 ; j++)
				outVRML << setw(20) << coordinate[inversenodeID[part_ND[i]-1]][j] * vSCALE_N;
			if (i <= (number_ND-2)) outVRML << "," << endl;
			if (i == (number_ND-1)) outVRML << endl;
		}
	} // end of caseNUMBER == 5

	else if (caseNUMBER == 6)
	{
		outVRML << "			]" << endl; // # keyValue
		outVRML << "		}" << endl; // # CoordinateInterpolator
	} // end of caseNUMBER == 6

	else if (caseNUMBER == 7)
	{
		outVRML << "	]" << endl;  //# children END
		outVRML << "}" << endl;  //# Transform END

		outVRML << "# blank for switch" << endl;

		outVRML << "	]" << endl;  //# Choice END
		outVRML << "}" << endl << endl;  //# Switch END

	} // end of caseNUMBER == 7
}

// PartObject::printXML printout PartObject-case in *.X3D with x3dOption = 1
void PartObject::printXML(ofstream& outVRML, DoubleArrayPtr* coordinate, DoubleArrayPtr animationTIME, 
						  IntArrayPtr inversenodeID, int caseNUMBER, int NTIM, double vSCALE_N)
{
	// color_test = 1 : color setting at material node 
	// color_test = 2 : color setting at color node
	int color_test = 1;
	// surface_type_test = 1 : quadrilateral element default with conditional triangular element
	// surface_type_test = 2 : all triangular element
	int surface_type_test = 1;
	// fraction_type_test = 1 => consider keyfraction as fraction of step number = i/NTIM
	// fraction_type_test = 2 => consider keyfraction as fractoin of each animation time = animationTIME[i] / animationTIME[NTIM]
	int fraction_type_test = 2;

	if (caseNUMBER == 1) 
	{
		//#	BodySwitch Node #
		outVRML << "  <Switch  DEF=\"" << kpartNAME << "_BodySwitch\"  whichChoice=\"0\">" << endl; // Switch START
		outVRML << "      <Transform  DEF=\"" << kpartNAME << "\" >" << endl;
		outVRML << "          <TouchSensor  DEF=\"" << kpartNAME << "_BodyTouch\" />" << endl;

		// # Shape Node #
		outVRML << "          <Shape >" << endl;

	} // end of caseNUMBER == 1

	else if (caseNUMBER == 2) 
	{
		double xRED = 0.0, xGREEN = 0.0, xBLUE = 0.0;
		setCOLOR(kPAT, xRED, xGREEN, xBLUE);

		//# appearence Node #
		if (color_test == 1) // color setting at material node
		{
			outVRML << "              <Appearance >" << endl;
			if ((kTYPE == 1) || (kTYPE == 2)) // solid or shell element - face segment
			{
				outVRML << "                 <Material  "
						<< "transparency=\"0.0\" "
						<< "emissiveColor=\"0.0 0.0 0.0\" "
						<< "shininess=\"0.2\" "
					    << "diffuseColor=\"" << xRED << " " << xGREEN << " " << xBLUE <<"\""
						<< "specularColor=\"0.5 0.5 0.5\"/>" << endl;
			}
			else if (kTYPE == 3)  //beam element - line segment
			{
				outVRML << "                 <Material  "
						<< "emissiveColor=\"" << xRED << " " << xGREEN << " " << xBLUE << "\"" << endl;
			}
			outVRML << "              </Appearance>" << endl;
		}

	}  // end of caseNUMBER == 2

	else if (caseNUMBER == 3) 
	{
		//# coordIndex Node #
		if (kTYPE == 3) outVRML << "		<IndexedLineSet  coordIndex=\"" << endl; // beam element - line segment
		if (kTYPE == 2) outVRML << "		<IndexedFaceSet  coordIndex=\"" << endl;  // shell element - face segment with "solid FALSE"
		if (kTYPE == 1) outVRML << "		<IndexedFaceSet  coordIndex=\"" << endl;  // solid element - face segment with "solid TRUE"

		for (int i = 0; i < number_SF; i++) 
		{
		    outVRML << "						";
		   if (part_SF[i][1] == part_SF[i][2]) // from beam element
			{
				for (int j = 0; j < 2; j++)
					outVRML << part_SF[i][j] << " ";
			}
			else if (part_SF[i][0] == part_SF[i][1] ) // from wedge solid element
			{
				for (int j = 1; j < 4; j++)
					outVRML << part_SF[i][j] << " ";
			}
			else if  (part_SF[i][2] == part_SF[i][3] ) // from triangle shell/plate element
			{
				for (int j = 0; j < 3; j++)
					outVRML << part_SF[i][j] << " ";
			}
			else // from rectangle shell/plate element or rectangle faec of solid element
			{
				if (surface_type_test == 1) // quadrilateral element default
				{
					for (int j = 0; j < 4; j++)
						outVRML << part_SF[i][j] << " ";
				}
				else if (surface_type_test == 2) // all triangular element
				{
					outVRML << part_SF[i][0] << " " << part_SF[i][1] << " " << part_SF[i][2] << " -1 ";
					outVRML << part_SF[i][2] << " " << part_SF[i][3] << " " << part_SF[i][0] << " ";
				}
			}
			if (i <= (number_SF-2)) outVRML << "-1 " << endl;
			if (i == (number_SF-1)) outVRML << "-1\"";
		}

		if (kTYPE == 3) outVRML << "						>" << endl;  // beam element - line segment
		if (kTYPE == 2) outVRML << "						creaseAngle=\"0.5\" solid=\"false\">" << endl;  // shell element - face segment with "solid FALSE"
		if (kTYPE == 1) outVRML << "						creaseAngle=\"0.5\" solid=\"false\">" << endl;  // solid element - face segment with "solid TRUE"

		//# coordinate Node #
		outVRML << "                  <Coordinate  DEF=\"" << kpartNAME << "_Coordinate\"  point=\"" << endl;
		for (int i = 0; i < number_ND; i++)
		{
			outVRML << "						";
			for (int j = 0; j < 3 ; j++)
				outVRML << setw(20) << coordinate[inversenodeID[part_ND[i]-1]][j] * vSCALE_N;
			if (i <= (number_ND-2)) outVRML << "," << endl;
			if (i == (number_ND-1)) outVRML << "\"" << endl;
		}
		outVRML << "					/>" << endl;   // # coordinate
		if (kTYPE == 3) outVRML << "              </IndexedLineSet>" << endl; // beam element - line segment
		if (kTYPE == 2) outVRML << "              </IndexedFaceSet>" << endl;  // shell element - face segment with "solid FALSE"
		if (kTYPE == 1) outVRML << "              </IndexedFaceSet>" << endl;  // solid element - face segment with "solid TRUE"
		outVRML << "          </Shape>" << endl; // #geometry

	} // end of caseNUMBER ==3

	else if (caseNUMBER == 4)
	{
		double keyfraction;

		//# CoordinateInterpolator Node #
		outVRML << "          <CoordinateInterpolator  DEF=\"" << kpartNAME << "_Morph\""
				<< "          			   key=\"" << endl;
		//# key Node #
		for (int i = 0 ; i < NTIM+1 ; i++)
		{
			if (fraction_type_test == 1) keyfraction = double(i) / double (NTIM); // fraction based on step number	
			else if (fraction_type_test == 2) keyfraction = double (animationTIME[i] / animationTIME[NTIM]); // fraction based on animation time
			outVRML << keyfraction << " ";
			if ((i != 0) && ((i%10) == 0)) outVRML << endl << "			     ";
		}
		outVRML << "\" " << endl; //# key

		//# KeyValue Node #
		outVRML << "          			   keyValue=\"" << endl;

	} // end of caseNUMBER == 4

	else if (caseNUMBER == 5)
	{
		outVRML << endl;
		for (int i = 0; i < number_ND; i++)
		{
			outVRML << "          			   		";
			for (int j = 0; j < 3 ; j++)
				outVRML << setw(20) << coordinate[inversenodeID[part_ND[i]-1]][j] * vSCALE_N;
			if (i <= (number_ND-2)) outVRML << "," << endl;
			if (i == (number_ND-1)) outVRML << endl;
		}
	} // end of caseNUMBER == 5

	else if (caseNUMBER == 6)
	{
		outVRML << "          			   		\"/>" << endl; // # keyValue
	} // end of caseNUMBER == 6

	else if (caseNUMBER == 7)
	{
		outVRML << "      </Transform>" << endl;  //# Transform END
		outVRML << "  </Switch>" << endl << endl;  //# Switch END

	} // end of caseNUMBER == 7
}


void PartObject::DebugOUT(ofstream& debug, int debugOption)
{
	if (debugOption == 1)
	{
		debug << "in DebugOUT" << endl;
		debug << "kPAT, kpartNAME =" << kPAT << " " << kpartNAME << endl;
		debug << "number_SF=" << number_SF << endl;
		for (int i = 0; i < number_SF; i++)
		{
			for (int j =0; j < 4; j++) debug << part_SF[i][j] << "  ";
			debug << endl;
		}
		debug << "number_ND=" << number_ND << endl;
		for (int i = 0; i < number_ND; i++)	debug << part_ND[i] << endl;
		debug << endl;
	}
}