// makeDAT.cpp : Data Processing Step 0

#include "stdafx.h"

#include <fstream>
#include <iomanip>
#include <string>
#include <cstdlib>
#include <cstring>
#include <stdio.h>
#include <math.h>

using namespace std;

typedef int* IntArrayPtr;
typedef double* DoubleArrayPtr;
typedef string* StringArrayPtr;


//**************************************************************************
// makeDAT read *.DYN (& *.TME) and make *.DAT 
//   as input file for conversion program Data Processing Step 1
//**************************************************************************
void makeDAT(string& inputDynNAME, string& inputTmeNAME, string& outputNAME,
			 double criteriaValue, int sColorOption)
{

	// inputDynNAME : input *.DYN filename w/ extension 
	// inputTmeName : input displacement-history filename w/extension
	// outputNAME : output filename (*.DAT) w/o extension
	// criteriaValue : criteria value for element deletion (elongation) 
	// sColorOption : option 1: same color 2: different color for deleted part with original part

	ifstream inDYN, inTME;
	ofstream outDAT;
	string fileNAME, fileTAG;

	string dumSTRING;			// dummy string
	char dumC, dumCarray[49];	// dummy character
	int dumINT;					// dummy integer

//**debug printout file
	ofstream debug;
	debug.open("imsi.dbg");
	debug.setf(ios::fixed | ios::showpoint | ios::right);

	// file open for *.DYN
	inDYN.open(inputDynNAME.c_str());
	if (inDYN.fail()) 
	{
		debug << "inDYN file open fail !!" << endl;
		return;
	}

	// setting for keyword & indicator searching 
	StringArrayPtr *indicatorDYN = new StringArrayPtr[6];
	for (int i = 0; i < 6; i++)	indicatorDYN[i] = new string;
	*indicatorDYN[0] = "$                                  PART CARDS                                  $";
	*indicatorDYN[1] = "$                               NODE INFORMATION                               $";
	*indicatorDYN[2] = "$                                BEAM ELEMENTS                                 $";
	*indicatorDYN[3] = "$                                SHELL ELEMENTS                                $";
	*indicatorDYN[4] = "$                                SOLID ELEMENTS                                $";
	*indicatorDYN[5] = "$                                PART SET CARDS                                $";

	IntArrayPtr lengthKeyword = new int[6];
	StringArrayPtr *keyword = new StringArrayPtr[6];
	for (int i = 0; i < 6; i++)
		keyword[i] = new string;
	*keyword[0] = "PART", lengthKeyword[0] = 4;
	*keyword[1] = "NODE", lengthKeyword[1] = 4;
	*keyword[2] = "ELEMENT_BEAM", lengthKeyword[2] = 12;
	*keyword[3] = "ELEMENT_SHELL", lengthKeyword[3] = 13;
	*keyword[4] = "ELEMENT_SOLID", lengthKeyword[4] = 13;
	*keyword[5] = "SET_PART_LIST_TITLE", lengthKeyword[5] = 19;

	// setting for control parameter
	// NNOD : Number of NODal points (== maxinum node number)
	// NNOD_temp : number of total nodal points
	// NELE : Number of total ELEments
	// NPAT : Number of PArTs (== maxinum part number)
	// NPAT_temp : number of total parts
	// NSEC : Number of total SECtions (segment set)

	// NELE_BEAM : number of total BEAM elements
	// NELE_SHELL : number of total SHELL elements
	// NELE_SOLILD : number of total SOLID elements

	int NNOD = 0, NNOD_temp =0, NELE = 0, NPAT = 0, NPAT_temp = 0, NSEC = 0;
	int NELE_BEAM = 0, NELE_SHELL = 0, NELE_SOLID = 0;

	// start of [read data from inDYN] part (1)
	//*******************************************************************
	while (! inDYN.eof())
	{
		getline(inDYN,dumSTRING);

		if (dumSTRING[0] == '$') 
		{
			// calculate NPAT_temp
			if (dumSTRING.substr(0,80) == *indicatorDYN[0]) // PART CARDS
			{
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				while (dumSTRING.substr(1,lengthKeyword[0]) == *keyword[0])
				{
					getline(inDYN,dumSTRING);
					if (dumSTRING[0] == '$') getline(inDYN,dumSTRING);
					NPAT_temp ++;
					getline(inDYN,dumSTRING);
					getline(inDYN,dumSTRING);
					getline(inDYN,dumSTRING);
				}
				continue;
			}
			// calculate NODE_temp
			else if (dumSTRING.substr(0,80) == *indicatorDYN[1]) // NODE INFORMATION
			{
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				if (dumSTRING.substr(1,lengthKeyword[1]) == *keyword[1])
				{
					getline(inDYN,dumSTRING);
					getline(inDYN,dumSTRING);
					NNOD_temp ++;
					while (dumSTRING[0] != '$')
					{
						getline(inDYN,dumSTRING);
						NNOD_temp ++;
					}
					NNOD_temp --;
				}
				continue;
			}
			// calculate NELE_BEAM
			else if (dumSTRING.substr(0,80) == *indicatorDYN[2]) // BEAM ELEMENT
			{
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				NELE_BEAM ++;
				while (dumSTRING[0] != '$')
				{
					getline(inDYN,dumSTRING);
					NELE_BEAM ++;
				}
				NELE_BEAM --;
				continue;
			}
			// calculate NELE_SHELL
			else if (dumSTRING.substr(0,80) == *indicatorDYN[3]) // SHELL ELEMENT
			{
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				NELE_SHELL ++;
				while (dumSTRING[0] != '$')
				{
					getline(inDYN,dumSTRING);
					NELE_SHELL ++;
				}
				NELE_SHELL --;
				continue;
			}
			// calculate NELE_SOLID
			else if (dumSTRING.substr(0,80) == *indicatorDYN[4])   // SOLID ELEMENT
			{
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				NELE_SOLID ++;
				while (dumSTRING[0] != '$')
				{
					getline(inDYN,dumSTRING);
					NELE_SOLID ++;
				}
				NELE_SOLID --;
				continue;
			}
			// calculate NSEC
			else if (dumSTRING.substr(0,80) == *indicatorDYN[5]) // PART SET CARDS
			{
				continue; // under construction
			}
		}
	}

	NELE = NELE_BEAM + NELE_SHELL + NELE_SOLID;
	//*******************************************************************
	// end of [read data from inDYN] part (1)


	// close and reopen *.DYN
	inDYN.close();
	inDYN.clear();
	inDYN.open(inputDynNAME.c_str());


	// setting for storage of part id & name in array
	IntArrayPtr partID = new int[NPAT_temp];
	StringArrayPtr *partNAME = new StringArrayPtr[NPAT_temp];
	for (int i = 0; i < NPAT_temp; i++)
		partNAME[i] = new string;

	// setting for storage of node id & coordinate in array
	IntArrayPtr nodeID = new int[NNOD_temp];
	DoubleArrayPtr *coordinate = new DoubleArrayPtr[NNOD_temp];
	for (int i = 0; i < NNOD_temp; i++) coordinate[i] = new double[3];

	// setting for storage of element id & connectivity in array
	IntArrayPtr elementID = new int[NELE];
	IntArrayPtr *element = new IntArrayPtr[NELE];
	for (int i = 0; i < NELE; i++) element[i] = new int[10];


	// start of [read data from inDYN] part (2)
	//*******************************************************************

	// iELE : internal element id used inside conversion program
	int iELE = 0;
	while (! inDYN.eof())
	{
		getline(inDYN,dumSTRING);

		if (dumSTRING[0] == '$') 
		{
			// store partID and partNAME
			if (dumSTRING.substr(0,80) == *indicatorDYN[0]) // PART CARDS
			{
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				for (int i = 0; i < NPAT_temp; i++)
				{
					getline(inDYN,dumSTRING);
					inDYN >> *partNAME[i];
					if (*partNAME[i] == "$HEADING") inDYN >> *partNAME[i];

					getline(inDYN,dumSTRING);
					getline(inDYN,dumSTRING);

					inDYN >> partID[i];

					getline(inDYN,dumSTRING);

					if (partID[i] > NPAT) NPAT = partID[i];
				}
				if (NPAT == 1) break;
				continue;
			}
			// store nodeID and coorinate
			else if (dumSTRING.substr(0,80) == *indicatorDYN[1]) // NODE INFORMATION
			{
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				if (dumSTRING.substr(1,lengthKeyword[1]) == *keyword[1])
				{
					getline(inDYN,dumSTRING);
					for (int i = 0; i < NNOD_temp ; i++)
					{
						inDYN >> nodeID[i];
						inDYN >> coordinate[i][0] >> coordinate[i][1] >> coordinate[i][2];
						getline(inDYN,dumSTRING);
						if (nodeID[i] > NNOD) NNOD = nodeID[i];
					}
				}
				continue;
			}
			// store beam element data in array elementID & element
			else if (dumSTRING.substr(0,80) == *indicatorDYN[2]) // BEAM ELEMENT
			{
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				for (int i = 0; i < NELE_BEAM; i++)
				{
					element[iELE][0] = 2;
					inDYN >> dumINT >> element[iELE][1] >> element[iELE][2] >> element[iELE][3];
					getline(inDYN,dumSTRING);
					iELE ++;
				}
				continue;
			}
			// store shell element data in array elementID & element
			else if (dumSTRING.substr(0,80) == *indicatorDYN[3]) // SHELL ELEMENT
			{
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				for (int i = 0; i < NELE_SHELL; i++)
				{
					element[iELE][0] = 4;
					inDYN >> dumINT >> element[iELE][1] >> element[iELE][2] >> element[iELE][3] >> element[iELE][4] >> element[iELE][5];
					elementID[iELE] = dumINT;
					getline(inDYN,dumSTRING);
					iELE ++;
				}
				continue;
			}
			// store solid element data in array elementID & element
			else if (dumSTRING.substr(0,80) == *indicatorDYN[4]) // SOLID ELEMENT
			{
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				getline(inDYN,dumSTRING);
				for (int i = 0; i < NELE_SOLID; i++)
				{
					inDYN >> dumINT;
					for (int j = 1; j < 10; j++)
						inDYN >> element[iELE][j];
					getline(inDYN,dumSTRING);
					if (element[iELE][6] == element[iELE][7])
					{
						element[iELE][0] = 6;
					}
					else
					{
						element[iELE][0] = 8;
					}
					iELE ++;
				}
				continue;
			}
			// store section (segment set) data
			else if (dumSTRING.substr(0,80) == *indicatorDYN[5]) // PART SET CARDS
			{
				continue; // under construction
			}
		}
	}

	//*******************************************************************
	// end of [read data from inDYN] part (2)

	// close *.DYN 
	inDYN.close();
	inDYN.clear();

	// setting for storage of node existancd flag & inversenodeID in array
	IntArrayPtr nodeEXIST = new int[NNOD];
	IntArrayPtr inversenodeID = new int[NNOD];
	int NNOD_real = 0;
	// check unused node & calculate really used node number
	for (int i = 0; i < NNOD; i++)
		nodeEXIST[i] = 0;
	for (int i = 0; i < NNOD_temp; i++)
		inversenodeID[nodeID[i]-1] = i;
	for (int i = 0; i < NELE; i++)
	{	
		int dumINT = element[i][0];
		if (dumINT == 6) dumINT = 8;
		for (int j = 0; j < dumINT; j++)
		{
			nodeEXIST[element[i][2+j]-1]++;
		}
	}
	for (int i = 0; i < NNOD; i++)
		if (nodeEXIST[i] > 0) NNOD_real++;

	if (inputTmeNAME.length() == 0)
	{
		// print out in *.DAT
		//********************************************
		fileTAG = ".DAT";
		fileNAME = outputNAME + fileTAG;
		outDAT.open(fileNAME.c_str());
		if (outDAT.fail()) 
			debug << "outDAT file open fail !!" << endl;
	
		outDAT << "** CONTROL PARAMETERS" << endl;
		outDAT << "   NNOD, NNOD_temp, NELE, NPAT, NPAT_temp, NTIM,  NSEC" << endl;
		outDAT << " " << NNOD << " " << NNOD_real << " " << NELE << " " << NPAT << " " << NPAT_temp << "   0   " << NSEC << endl;

		outDAT << "** NODE INFORMATION" << endl;
		for (int i = 0; i < NNOD; i++) 
		{	
			if (nodeEXIST[i] > 0)
			{
				outDAT << setw(10) << nodeID[inversenodeID[i]];
				for (int j = 0; j < 3; j++)
					outDAT << setw(15) << coordinate[inversenodeID[i]][j];
				outDAT << endl;
			}
		}

		outDAT << "** ELEMENT INFORMATION" << endl;
		for (int i = 0; i < NELE; i++)
		{
			if (element[i][0] == 2) 
				for (int j = 0; j < 4; j++) outDAT << " " << element[i][j];	
			else if (element[i][0] == 4)
				for (int j = 0; j < 6; j++) outDAT << " " << element[i][j];	
			else
				for (int j = 0; j < 10; j++) outDAT << " " << element[i][j]; 
			outDAT << endl;
		}

		outDAT << "** PART INFORMATION" << endl;
		for (int i = 0; i < NPAT_temp; i++)
			outDAT << " " << *partNAME[i] << setw(10) << partID[i] << "   0  " << endl;

		outDAT << "** TIME SPACING INFORMATION" << endl;
		outDAT << "** SET for PARTS INFORMATION : blank / sameColor / (texture URL)" << endl;
		outDAT.close();
		//********************************************

		// delete array for first part

		delete[] indicatorDYN;
		delete[] lengthKeyword;
		delete[] keyword;

		delete[] partID;
		delete[] partNAME;

		delete[] nodeID;
		for (int i = 0; i < NNOD_temp; i++)
			delete[] coordinate[i];
		delete[] coordinate;

		delete[] elementID;
		for (int i = 0; i < NELE; i++)
			delete[] element[i];
		delete[] element;

		delete[] nodeEXIST;
		delete[] inversenodeID;

		return;
	}


	// file open for *.DYN
	inTME.open(inputTmeNAME.c_str());
	if (inTME.fail()) 
	{
		debug << "inTME file open fail !!" << endl;
		delete[] indicatorDYN;
		delete[] lengthKeyword;
		delete[] keyword;

		delete[] partID;
		delete[] partNAME;

		delete[] nodeID;
		for (int i = 0; i < NNOD_temp; i++)
			delete[] coordinate[i];
		delete[] coordinate;

		delete[] elementID;
		for (int i = 0; i < NELE; i++)
			delete[] element[i];
		delete[] element;

		delete[] nodeEXIST;
		delete[] inversenodeID;
		return;
	}

	// start of [read data from inTME] part (1)
	//*******************************************************************
	// maxinum of time step = 100
	DoubleArrayPtr inputTIME = new double[100];
	int NTIM = 0;
	while (! inTME.eof())
	{
		getline(inTME,dumSTRING);
		if (dumSTRING.substr(0,8) == "*KEYWORD")
		{
			inTME.get(dumCarray,14);
			inTME >> inputTIME[NTIM];
			NTIM ++;
		}
	}
	inTME.close();
	inTME.clear();


	// element deletion check routine start
	//****************************************************************

	// setting for deleted element ID & deleted time
	IntArrayPtr elementDELE = new int[NELE];
	for (int i = 0; i < NELE; i++)
		elementDELE[i] = 0;
	DoubleArrayPtr elementDELEtime = new double[NELE];

	// setting for deformed coordinate;
	DoubleArrayPtr *deformed = new DoubleArrayPtr[NNOD_temp];
	for (int i = 0; i < NNOD_temp; i++) deformed[i] = new double[3];

	// node1 : start node number for deformation calculation
	// node2 : end node number for deformation calculation
	int node1,node2;
	// Lmax   : deformed length from node1 to node2 
	// Lmin   : undeformed length from node1 to node2
	// indicator = Lmax / Lmin
	double Lmax, Lmin, indicator, xx,yy,zz;

	// file open for *.TME
	inTME.open(inputTmeNAME.c_str());

	int DELE = 0;
	for (int iTIM = 0; iTIM < NTIM; iTIM++)
	{
		getline(inTME,dumSTRING);
		getline(inTME,dumSTRING);
		getline(inTME,dumSTRING);
		getline(inTME,dumSTRING);
		for (int i = 0; i < NNOD; i++) 
		{
			inTME.get(dumC);
			if (dumC == '*') break;
			inTME >> dumINT >> xx >> yy >> zz;
			getline(inTME,dumSTRING);

			if ((inversenodeID[dumINT-1] < 0) || (inversenodeID[dumINT-1] >= NNOD_temp)) continue;

			deformed[inversenodeID[dumINT-1]][0] = coordinate[inversenodeID[dumINT-1]][0]  + xx;
			deformed[inversenodeID[dumINT-1]][1] = coordinate[inversenodeID[dumINT-1]][1]  + yy;
			deformed[inversenodeID[dumINT-1]][2] = coordinate[inversenodeID[dumINT-1]][2]  + zz;
		}
		getline(inTME,dumSTRING);

		// calculate element characteristic elongation
		for (int i = 0; i < NELE; i++) 
		{
			indicator = 0.0;
			if (elementDELE[i] == 1) continue;

			// beam element
			if (element[i][0] == 2)
			{
				Lmax = 0.0, Lmin = 0.0;
				for (int j = 0; j < 3; j++)
				{
					node1 = inversenodeID[element[i][2]-1];
					node2 = inversenodeID[element[i][3]-1];
					Lmin = Lmin + pow ((coordinate[node1][j] - coordinate[node2][j]), 2);
					Lmax = Lmax + pow ((deformed[node1][j] - deformed[node2][j]), 2);
				}
				Lmax = pow (Lmax, 0.5);
				Lmin = pow (Lmin, 0.5);
				if (Lmin != 0.0) indicator = Lmax / Lmin;
			}

			// shell element - diagonal elongation check
			else if (element[i][0] == 4)
			{
				for (int iNod = 0; iNod < 2; iNod++)
				{
					node1 = inversenodeID[element[i][iNod+2]-1];
					node2 = inversenodeID[element[i][(iNod+2)+2]-1];

					Lmax = 0.0, Lmin = 0.0;
					for (int j = 0; j < 3; j++)
					{
						Lmin = Lmin + pow ((coordinate[node1][j] - coordinate[node2][j]), 2);
						Lmax = Lmax + pow ((deformed[node1][j] - deformed[node2][j]), 2);
					}
					if ((Lmax / Lmin) > indicator) indicator = (Lmax / Lmin);
				}
			}

			// solid element
			else if (element[i][0] == 8)
			{
				for (int iNod = 0; iNod < 4; iNod++)
				{
					node1 = inversenodeID[element[i][iNod+2]-1];
					if (iNod < 2) node2 = inversenodeID[element[i][(iNod+2)+6]-1];
					if (iNod >= 2) node2 = inversenodeID[element[i][(iNod+2)+2]-1];

					Lmax = 0.0, Lmin = 0.0;
					for (int j = 0; j < 3; j++)
					{
						Lmin = Lmin + pow ((coordinate[node1][j] - coordinate[node2][j]), 2);
						Lmax = Lmax + pow ((deformed[node1][j] - deformed[node2][j]), 2);
					}
					if ((Lmax / Lmin) > indicator) indicator = (Lmax / Lmin);
				}

			}
			if (indicator > criteriaValue) // indicator for element deletion
			{
				elementDELE[i] = 1;
				elementDELEtime[i] = inputTIME[iTIM];
				DELE++;
			}
		}
	}

	//****************************************************************
	// element deletion check routine end


	// devide deleted elements as a part at each time step -- start
	//***************************************************************

	// setting for deleted part data at each time step
	IntArrayPtr *deletePART = new IntArrayPtr[NPAT_temp];
	for (int i = 0; i < NPAT_temp; i++) deletePART[i] = new int[NTIM];
	for (int i = 0; i < NPAT_temp; i++)
		for (int j = 0; j < NTIM; j++)
			deletePART[i][j] = 0;

	// search deleted element and store this element data in deletePART array
	//-----------------------------------------------------------------------------------------
	//	1. element((elementDELE[i] == 1)) is deleted detween inputTIME[k-1] and inputTIME[k]
	//  2. addition process is occured with part of this deleted element at according time step
	//  3. finally each part have the number of deleted elements at each time step in deletePART[partID][inputTIME]
	//--------------------------------------------------------------------------------------------------------------
	for (int i = 0; i < NELE; i++)
	{
		if (elementDELE[i] == 1)
		{
			for (int k = 0; k < NTIM; k++)
			{
				if (((k == 0) && (elementDELEtime[i] <= inputTIME[k])) ||
					((k > 0) && (elementDELEtime[i] > inputTIME[k-1]) && (elementDELEtime[i] <= inputTIME[k])))
				{
					for (int l = 0; l < NPAT_temp; l++)
					{
						if (element[i][1] == partID[l])
						{
							deletePART[l][k]++;
						}
					}
				}
			}
		}		
	}

	// store divided deleted_element_part ID to element array
	//-----------------------------------------------------------------------------------------
	//	1. each part search their deleted element at each time step using partID & elementDELE array
	//  2. deleted elements with same partID & deleted time step assebled as one part with partID (NPAT + NPAT_delete)
	//--------------------------------------------------------------------------------------------------------------
	int NPAT_delete = 0;
	for (int i = 0; i < NPAT_temp; i++)
	{
		for (int j = 0; j < NTIM; j++)
		{
			if (deletePART[i][j] > 0) 
			{
				NPAT_delete++;
				for (int k = 0; k < NELE; k++)
				{
					if ((element[k][1] == partID[i]) && (elementDELE[k] == 1))
					{
						for (int m = 0; m < NTIM; m++)
						{

							if (((m == 0) && (elementDELEtime[k] <= inputTIME[m])) ||
								((m > 0) && (elementDELEtime[k] > inputTIME[m-1]) && (elementDELEtime[k] <= inputTIME[m])))
							{
								element[k][1] = NPAT + NPAT_delete;
							}
						}
					}
				}
			}
		}
	}

	// assembled undeleted part & deleted part with same partID as one seciont(segment set)
	int NSEC_divide = 0;
	for (int i = 0; i < NPAT_temp; i++)
	{
		int FLAG_divide = 0;
		for (int j = 0; j < NTIM; j++)
			if (deletePART[i][j] > 0) FLAG_divide ++;
		if (FLAG_divide > 0) NSEC_divide++;
	}

	//***************************************************************
	// devide deleted elements as a part at each time step -- end


	// print out in *.DAT
	//********************************************
	fileTAG = ".DAT";
    fileNAME = outputNAME + fileTAG;
	outDAT.open(fileNAME.c_str());
	if (outDAT.fail()) 
		debug << "outDAT file open fail !!" << endl;
	
	outDAT << "** CONTROL PARAMETERS" << endl;
	outDAT << "   NNOD, NNOD_temp, NELE, NPAT, NPAT_temp, NTIM,  NSEC" << endl;
	outDAT << " " << NNOD << " " << NNOD_real << " " << NELE << " " << (NPAT+NPAT_delete) << " " << (NPAT_temp+NPAT_delete) << " " << NTIM << " " << (NSEC+NSEC_divide) << endl;

	outDAT << "** NODE INFORMATION" << endl;
	for (int i = 0; i < NNOD; i++) 
	{	
		if (nodeEXIST[i] > 0)
		{
			outDAT << setw(10) << nodeID[inversenodeID[i]];
			for (int j = 0; j < 3; j++)
				outDAT << setw(15) << coordinate[inversenodeID[i]][j];
			outDAT << endl;
		}
	}

	outDAT << "** ELEMENT INFORMATION" << endl;
	for (int i = 0; i < NELE; i++)
	{
		if (element[i][0] == 2) 
			for (int j = 0; j < 4; j++) outDAT << " " << element[i][j];	
		else if (element[i][0] == 4)
			for (int j = 0; j < 6; j++) outDAT << " " << element[i][j];	
		else
			for (int j = 0; j < 10; j++) outDAT << " " << element[i][j]; 
		outDAT << endl;
	}

	outDAT << "** PART INFORMATION" << endl;
	int IPAT_delete = 0;
	for (int i = 0; i < NPAT_temp; i++)
	{
		outDAT << " " << *partNAME[i] << setw(10) << partID[i] << "   0  " << endl;
		for (int j = 0; j < NTIM; j++)
		{
			if (deletePART[i][j] > 0) 
			{
				IPAT_delete ++;
				// undeleted part
				if (j == 0)
					outDAT << " " << *partNAME[i] << "_ts" << j << setw(10) << (NPAT + IPAT_delete) << "    1  " << "  0.0" << endl;
				// deleted part at specific time step of inputTIME[j-1]
				else 
					outDAT << " " << *partNAME[i] << "_ts" << j << setw(10) << (NPAT + IPAT_delete) << "    1  " << inputTIME[j-1] << endl;
			}
		}
	}

	outDAT << "** TIME SPACING INFORMATION" << endl;
	for (int i = 0; i < NTIM; i++)
		outDAT << " " << (i+1) << "   1" << endl;

	outDAT << "** SET for PARTS INFORMATION : blank / sameColor / (texture URL)" << endl;
	IPAT_delete = 0;
	int ISEC_divide = 0;
	for (int i = 0; i < NPAT_temp; i++)
	{
		int FLAG_divide = 0;
		for (int j = 0; j < NTIM; j++)
			if (deletePART[i][j] > 0) FLAG_divide ++;
		if (FLAG_divide > 0) 
		{
			NSEC_divide++;
			if (sColorOption == 1)
				outDAT << (FLAG_divide+1) << "    " << *partNAME[i] << "_original" << "   " << "defaultSameColor" << endl;
			if (sColorOption == 2)
				outDAT << (FLAG_divide+1) << "    " << *partNAME[i] << "_original" << "   " << "blank" << endl;
			outDAT << partID[i];
            for (int j = 0; j < FLAG_divide; j++)
			{
				IPAT_delete ++;
				outDAT << " " << (NPAT+IPAT_delete);
			}
			outDAT << endl;
		}
	}
	outDAT.close();

	// delete array for first part

	delete[] indicatorDYN;
	delete[] lengthKeyword;
	delete[] keyword;

	delete[] partID;
    delete[] partNAME;

	delete[] nodeID;
	for (int i = 0; i < NNOD_temp; i++)
		delete[] coordinate[i];
	delete[] coordinate;

	delete[] elementID;
	for (int i = 0; i < NELE; i++)
		delete[] element[i];
	delete[] element;

	// delete array for second part

	delete[] inputTIME;

	delete[] elementDELE;

	delete[] nodeEXIST;
	delete[] inversenodeID;

	for (int i = 0; i < NNOD_temp; i++)
		delete[] deformed[i];
	delete[] deformed;

	for (int i = 0; i < NPAT_temp; i++)
		delete[] deletePART[i];
	delete[] deletePART;

	return;
}