// makeWRL.cpp : Data Processing Step 1

#include "stdafx.h"

#include <fstream>
#include <iomanip>
#include <string>
#include <cstdlib>
#include <cstring>
#include <math.h>
#include <stdio.h>

#include "PartObject.h"
#include "SectionObject.h"

using namespace std;

typedef int* IntArrayPtr;
typedef double* DoubleArrayPtr;
typedef double** DoubleArray2Ptr;
typedef string* StringArrayPtr;
typedef class PartObject* PartArrayPtr;
typedef class SectionObject* SectionArrayPtr;

// NNofFACE[FACE][CORNER]: Node Connectivity for Each Face
const int NNofFACE[6][4] = {4,1,5,8,  2,3,7,6,  1,2,6,5,
							3,4,8,7,  1,4,3,2,  5,6,7,8};

// getDIMEN : calculate maximum length of each x,y,z direction roughly
void getDIMEN(DoubleArrayPtr* coordinate, double dimension[][3], int NNOD_temp);

// extractSURFX : extract surface data for solid element
void extractSURFX(IntArrayPtr* element,IntArrayPtr* e_surfx, int NELE);

// getSIZE : calculate the number of Surface(size_SF) & Node Numbers(size_ND) for each part
void getSIZE(IntArrayPtr* element, IntArrayPtr* e_surfx,
			  IntArrayPtr tempNODE, IntArrayPtr tempSURFACE,
			  int NELE, int kPAT, int& kTYPE, int& size_SF, int& size_ND);

// printWRL : printout Overall-case data in *.WRL
void printWRL(ofstream& outWRL, StringArrayPtr *partNAME, 
			  int NPAT, int NPAT_temp, int caseNUMBER, double cycleINTERVAL, int aOption,
			  double vSCALE_N, double dimension[][3],
			  IntArrayPtr flagDELETE, DoubleArrayPtr timeDELETE, double endTIME);

// printX3D : printout Overall-case data in *.X3D
void printX3D(ofstream& outWRL, StringArrayPtr *partNAME, 
			  int NPAT, int NPAT_temp, int caseNUMBER, double cycleINTERVAL, int aOption,
			  double vSCALE_N, double dimension[][3], string& fileNAME);

// getHISTORY : read displacement and calculate deformed shape and store it in history_data at each animationTIME step
void getHISTORY(ifstream& inTME, ofstream& debug,IntArrayPtr inversenodeID,
			   DoubleArrayPtr* undeformed, DoubleArray2Ptr* history_data, 
			   DoubleArrayPtr animationTIME, IntArrayPtr spacingTIME,
			   int NNOD, int NNOD_temp, int NTIM, double vSCALE_D);


//*****************************************************************
// makeWRL read *.DAT and make *.WRL as input file for VRML browser
//*****************************************************************
void makeWRL(string& inputTmeNAME, string& outputNAME, int aOption, 
			 double vSCALE_D, double vSCALE_N, int x3dOption)
{

//===================================================================//
//                                                                   //
//          PROCESS <1> DATA READING from *.DAT                      //
//                                                                   //
//===================================================================//

	// inputTmeNAME : input history-displacement filename w/ extension
	// outputNAME : output filename (*.DAT, *.WRL/*.X3D) w/o extension
	// aOption : 1 static VRML (Geometry Only) & 2 dynamic static VRML (Simulation)
	// x3dOption : printout option for VRML = 1, X3D = 2
	// vSCALE_D; read Displacement Data and convert to Coordinate with scale
	// vSCALE_N; change the coordinate scale to fit walk/fly Navigation scale

	ifstream inDAT,inTME;
	ofstream outVRF,outHTM;
	string fileNAME, fileTAG;

	string dumSTRING, dumSTRING2;	// dummy string
	int dumINT, dumINT2;			// dummy integer

//**debug printout file
	ofstream debug;
	debug.open("imsi.dbg");
	debug.setf(ios::fixed | ios::showpoint | ios::right);

	// file open for *.DAT
	fileTAG = ".dat";
    fileNAME = outputNAME + fileTAG;
	inDAT.open(fileNAME.c_str());
	if (inDAT.fail()) 
		debug << "inDAT file open fail !!" << endl;

	// start of [read data from inDAT]
	//*******************************************************************

	// setting for control parameter
	//------------------------------
	// NNOD : Number of NODal points (== maxinum node number)
	// NNOD_temp : number of total nodal points
	// NELE : Number of total ELEments
	// NPAT : Number of PArTs (== maxinum part number)
	// NPAT_temp : number of total parts
	// NSEC : Number of total SECtions (segment set)
	// NTIM : Number of total TIMe steps

	int NNOD, NNOD_temp, NELE, NPAT, NPAT_temp, NTIM, NSEC;

	getline(inDAT,dumSTRING);
	getline(inDAT,dumSTRING);

	// read data part [1] - control data

	inDAT >> NNOD >> NNOD_temp >> NELE >> NPAT >> NPAT_temp >> NTIM >> NSEC;
	getline(inDAT,dumSTRING);

	// setting for storage of each array
	//----------------------------------
	// nodeID[NNOD_temp] : node index
	// inversenodeID[NNOD] : inverse of node index
	// undeformed[NNOD_temp][3] : undeformed coordinate x,y,z
	// coordinate[NNOD_temp][3] : deformed coordinate x,y,z
	// history_data[NTIM+1][NNOD_temp][3] : deformed coordinate x,y,z
	// element[NELE][i]  : element information kNOD, partID, connectivity
	// e_surfx[NELE][6] : surface data for solid element
	// tempNODE[NNOD_temp] : temporary storage array for Node Number for each Part
	// tempSURFACE[NELE*4] : temporary storage array for Element Connectivity for each Part
	// spacingTIME[NTIM] : time spacing data of printout for *.wrl
	// animationTIME[NTIM+1] : time data of printout for *.wrl
	// partNAME[NPAT] : name for each Part
	// flagDELETE[NPAT] : flag of deletion for each Part
	// timeDELETE[NPAT] : time of deletion for each Part
	// PartArrayPtr[NPAT] : PartObject Class
	// SectionArrayPtr[NSEC] : SectionObject Class

	IntArrayPtr nodeID = new int[NNOD_temp];
	IntArrayPtr inversenodeID = new int[NNOD];

	DoubleArrayPtr *undeformed = new DoubleArrayPtr[NNOD_temp];
	for (int i = 0; i < NNOD_temp; i++) undeformed[i] = new double[3];
	for (int i = 0; i < NNOD_temp; i++)
		for (int j = 0; j < 3; j++)
			undeformed[i][j] = 0.00000;

	DoubleArrayPtr *coordinate = new DoubleArrayPtr[NNOD_temp];
	for (int i = 0; i < NNOD_temp; i++) coordinate[i] = new double[3];

	DoubleArray2Ptr *history_data = new DoubleArray2Ptr[NTIM+1];
	for (int i = 0; i < (NTIM+1); i++) history_data[i] = new DoubleArrayPtr[NNOD_temp];
	for (int i = 0; i < (NTIM+1); i++) 
		for (int j = 0; j < NNOD_temp; j++)
			history_data[i][j] = new double[3];

	IntArrayPtr *element = new IntArrayPtr[NELE];
	for (int i = 0; i < NELE; i++) element[i] = new int[10];

	IntArrayPtr *e_surfx = new IntArrayPtr[NELE];
	for (int i = 0; i < NELE; i++) e_surfx[i] = new int[6];

	IntArrayPtr tempNODE = new int[NNOD_temp];
	IntArrayPtr tempSURFACE = new int[NELE*4];

	IntArrayPtr spacingTIME = new int[NTIM];
	DoubleArrayPtr animationTIME = new double[NTIM+1];

	StringArrayPtr *partNAME = new StringArrayPtr[NPAT];
	for (int i = 0; i < NPAT; i++)
		partNAME[i] = new string;

	IntArrayPtr flagDELETE = new int[NPAT];
	DoubleArrayPtr timeDELETE = new double[NPAT];

	PartArrayPtr *PartArray = new PartArrayPtr[NPAT];
	SectionArrayPtr *SectionArray = new SectionArrayPtr[NSEC];

	// read data part [2] - other data

	//**read NODE INFORMATION
	getline(inDAT,dumSTRING);
	for (int i = 0; i < NNOD_temp; i++) 
	{
		inDAT >> nodeID[i];
		for (int j = 0; j < 3; j++)
			inDAT >> undeformed[i][j];
	}
	getline(inDAT,dumSTRING);
	// set inversenodeID array
	for (int i = 0; i < NNOD_temp; i++)
		inversenodeID[nodeID[i]-1] = i;

	//**read ELEMENT INFORMATION
	getline(inDAT,dumSTRING);
	int kNOD;  // number of node for each element (2:beam or 4:shell or 6/8:solid)
	for (int i = 0; i < NELE; i++) 
	{
		inDAT >> kNOD;
		element[i][0] = kNOD;
		inDAT >> dumINT;
		element[i][1] = dumINT - 1;
		for (int j = 2; j < 10; j++)
		{
			if ((kNOD < 6) && (j > (2+kNOD-1))) continue;
			inDAT >> dumINT;
            element[i][j] = dumINT;
		}
		if ((kNOD == 8) && (element[i][8] == element[i][9])) element[i][0] = 6;
	}
	getline(inDAT,dumSTRING);

	//**read PART INFORMATION
	getline(inDAT,dumSTRING);

	for (int i = 0; i < NPAT_temp; i++)
	{
		inDAT >> dumSTRING >> dumINT >> dumINT2;

		for (int j = 0; j < signed (dumSTRING.length()); j++)
			if (dumSTRING[j] == '.') dumSTRING[j] = '_';
		*partNAME[dumINT-1] = "part_" + dumSTRING;

		flagDELETE[dumINT-1] = dumINT2;
		if (flagDELETE[dumINT-1] == 1) inDAT >> timeDELETE[dumINT-1];
	}
	getline(inDAT,dumSTRING);

	//**read TIME SPACING INFORMATION
	getline(inDAT,dumSTRING);
	for (int i = 0; i < NTIM; i++)
	{
		inDAT >> dumINT >> spacingTIME[i];
		if (spacingTIME[i] == 0) spacingTIME[i] = 1;
	}
	getline(inDAT,dumSTRING);

	//**read PARTSET INFORMATION
	getline(inDAT,dumSTRING);
	double dumColor[3] = {1,1,1};
	for (int i = 0; i < NSEC; i++)
	{
		inDAT >> dumINT >> dumSTRING >> dumSTRING2;
		if (dumSTRING2 == "givenSameColor") {inDAT >> dumColor[0] >> dumColor[1] >> dumColor[2];}
		SectionArray[i] = new SectionObject(dumINT);
		SectionArray[i]->setkSEC(i, dumSTRING, dumSTRING2, dumColor);
		for (int j = 0; j < (SectionArray[i]->getNumPT()); j++)
		{
			inDAT >> dumINT;
			SectionArray[i]->setARRAY(j, (dumINT-1));
		}
	}

	//*******************************************************************
	// end of [read data from inDAT]

	inDAT.close();

//===================================================================//
//                                                                   //
//       PROCESS <2> SEGMENTATION                                    //
//                                                                   //
//===================================================================//

	// start with undeformed geometry to calculate whole dimension
	double dimension[2][3] = {0,0,0, 0,0,0};
	getDIMEN(undeformed, dimension, NNOD_temp);

	// change vSCALE_N for [maxLength => user-input vSCALE_N value]
	vSCALE_N = vSCALE_N / pow ((dimension[0][0] * dimension[0][1] * dimension[0][2]), (1/3.));

	// function extractSURFX : solid element -> surface extraction
	for (int i = 0; i < NELE; i++) 
		for (int j = 0; j < 6; j++)
			e_surfx[i][j]=0;
	extractSURFX(element, e_surfx, NELE);

	// definition of PartObject class
	int iTYPE, size_SF, size_ND;
	for (int i = 0; i < NPAT; i++)
	{
		dumSTRING = *partNAME[i];
		if (dumSTRING.empty()) continue;

		// calculate the number of SurFace(size_SF) nad NoDe index(size_ND) for each part
        getSIZE(element, e_surfx, tempNODE, tempSURFACE, NELE, i, iTYPE, size_SF, size_ND);
		PartArray[i] = new PartObject(size_SF, size_ND);
		PartArray[i]->setkPAT(i, iTYPE, *partNAME[i]);
		PartArray[i]->setARRAY(tempNODE, tempSURFACE);
		PartArray[i]->DebugOUT(debug,1);
	}

//===================================================================//
//                                                                   //
//       PROCESS <3> DISPLACEMENT READING from *.TME & PRINTING      //
//                                                                   //
//===================================================================//

	// file open for output file
	if (x3dOption == 1) fileTAG = ".wrl";
	if (x3dOption == 2) fileTAG = ".x3d";
	fileNAME = outputNAME + fileTAG;
	outVRF.open(fileNAME.c_str());

	// [start of aOption = 1 : static VRML (Geometry Only) ]
	//******************************************************
	if (aOption == 1) 
	{
		// printout part[1] of Overall-case with caseNUMBER=1]
		if (x3dOption == 1) printWRL(outVRF, partNAME, NPAT, NPAT_temp, 1, 10, aOption, vSCALE_N, dimension, flagDELETE, timeDELETE, animationTIME[NTIM]);
		if (x3dOption == 2) printX3D(outVRF, partNAME, NPAT, NPAT_temp, 1, 10, aOption, vSCALE_N, dimension, fileNAME);

		// printout process for defined sections first
		for (int kSEC = 0; kSEC < NSEC; kSEC++)
		{
			for (int kPT = 0; kPT < (SectionArray[kSEC]->getNumPT()); kPT++)
			{
				int kPAT = (SectionArray[kSEC]->getSecPT(kPT));
				PartArray[kPAT]->setSECTION(1);

				if (x3dOption == 1) // WRL
				{
					// printout PartObject-case with caseNUMBER=1]
					// printout SectionObject-case with caseNUMBER=3]
					// printout PartObject-case with caseNUMBER=3,7]
					PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 1, NTIM,vSCALE_N);
					SectionArray[kSEC]->printVRML(outVRF, 3, kPAT, (PartArray[kPAT]->getkTYPE()));
					PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 3, NTIM, vSCALE_N);
					PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 7, NTIM, vSCALE_N);
				}
				else if (x3dOption == 2) // X3D
				{
					// printout PartObject-case with caseNUMBER=1]
					// printout SectionObject-case with caseNUMBER=3]
					// printout PartObject-case with caseNUMBER=3,7]
					PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 1, NTIM, vSCALE_N);
					SectionArray[kSEC]->printXML(outVRF, 3, kPAT, (PartArray[kPAT]->getkTYPE()));
					PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 3, NTIM, vSCALE_N);
					PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 7, NTIM, vSCALE_N);
				}
			}
		}
		// printout process for remained parts later
		for (int kPAT = 0; kPAT < NPAT; kPAT++)
		{
			dumSTRING = *partNAME[kPAT];
			if (dumSTRING.empty()) continue;
			if ((PartArray[kPAT]->getSECTION()) == 1) continue;
			
			// printout PartObject-case with caseNUMBER=1,2,3,7]
			if (x3dOption == 1) // WRL
			{
				PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 1, NTIM, vSCALE_N);
				PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 2, NTIM, vSCALE_N);
				PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 3, NTIM, vSCALE_N);
				PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 7, NTIM, vSCALE_N);
			}
			else if (x3dOption == 2) // X3D
			{
				PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 1, NTIM, vSCALE_N);
				PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 2, NTIM, vSCALE_N);
				PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 3, NTIM, vSCALE_N);
				PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 7, NTIM, vSCALE_N);
			}
		}

		// printout part[1] of Overall-case with caseNUMBER=2]
		if (x3dOption == 1) printWRL(outVRF, partNAME, NPAT, NPAT_temp, 2, 10, aOption, vSCALE_N, dimension, flagDELETE, timeDELETE, animationTIME[NTIM]);
		if (x3dOption == 2) printX3D(outVRF, partNAME, NPAT, NPAT_temp, 2, 10, aOption, vSCALE_N, dimension, fileNAME);

		outVRF.close();
	}
	//****************************************************
	// [end of aOption = 1 : static VRML (Geometry Only) ]

	// [start of aOption = 2 : dynamic static VRML (Simulation) ]
	//***********************************************************
	if (aOption == 2)
	{
		// read displacement and calculate deformed shape and store it in history_data at each animationTIME step
		inTME.open(inputTmeNAME.c_str());
		if (inTME.fail()) debug << "inTME file open fail !!" << endl;
		getHISTORY(inTME, debug, inversenodeID, undeformed, history_data,
			animationTIME, spacingTIME,	NNOD, NNOD_temp, NTIM, vSCALE_D);
		inTME.close();

		// printout part[1] of Overall-case with caseNUMBER=1]
		if (x3dOption == 1) printWRL(outVRF, partNAME, NPAT, NPAT_temp, 1, animationTIME[NTIM], aOption, vSCALE_N, dimension, flagDELETE, timeDELETE, animationTIME[NTIM]);
		if (x3dOption == 2) printX3D(outVRF, partNAME, NPAT, NPAT_temp, 1, animationTIME[NTIM], aOption, vSCALE_N, dimension, fileNAME);

		// printout process for defined sections first
		for (int kSEC = 0; kSEC < NSEC; kSEC++)
		{
			for (int kPT = 0; kPT < (SectionArray[kSEC]->getNumPT()); kPT++)
			{
				int kPAT = (SectionArray[kSEC]->getSecPT(kPT));
				PartArray[kPAT]->setSECTION(1);

				if (x3dOption == 1) // WRL
				{
					// printout PartObject-case with caseNUMBER=1,(2)->changed in SectionObject-case,3,4]
					PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 1, NTIM, vSCALE_N);
					SectionArray[kSEC]->printVRML(outVRF, 3, kPAT, (PartArray[kPAT]->getkTYPE()));
					PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 3, NTIM, vSCALE_N);
					PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 4, NTIM, vSCALE_N);
					// read coordinate at each time step
					for (int iTIM = 0; iTIM < NTIM+1; iTIM++)
					{
						for (int iNOD = 0; iNOD < NNOD_temp; iNOD++)
							for (int j = 0; j < 3; j++)
								coordinate[iNOD][j] = history_data[iTIM][iNOD][j];
						// printout PartObject-case with caseNUMBER=5]
						PartArray[kPAT]->printVRML(outVRF, coordinate, animationTIME, inversenodeID, 5, NTIM, vSCALE_N);
					}
					// printout PartObject-case with caseNUMBER=6,7]
					PartArray[kPAT]->printVRML(outVRF, coordinate, animationTIME, inversenodeID, 6, NTIM, vSCALE_N);
					PartArray[kPAT]->printVRML(outVRF, coordinate, animationTIME, inversenodeID, 7, NTIM, vSCALE_N);
				}
				else if (x3dOption == 2)
				{
					// printout PartObject-case with caseNUMBER=1,(2)->changed in SectionObject-case,3,4]
					PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 1, NTIM, vSCALE_N);
					SectionArray[kSEC]->printXML(outVRF, 3, kPAT, (PartArray[kPAT]->getkTYPE()));
					PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 3, NTIM, vSCALE_N);
					PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 4, NTIM, vSCALE_N);
					// read coordinate at each time step
					for (int iTIM = 0; iTIM < NTIM+1; iTIM++)
					{
						for (int iNOD = 0; iNOD < NNOD_temp; iNOD++)
							for (int j = 0; j < 3; j++)
								coordinate[iNOD][j] = history_data[iTIM][iNOD][j];
						// printout PartObject-case with caseNUMBER=5]
						PartArray[kPAT]->printXML(outVRF, coordinate, animationTIME, inversenodeID, 5, NTIM, vSCALE_N);
					}
					// printout PartObject-case with caseNUMBER=6,7]
					PartArray[kPAT]->printXML(outVRF, coordinate, animationTIME, inversenodeID, 6, NTIM, vSCALE_N);
					PartArray[kPAT]->printXML(outVRF, coordinate, animationTIME, inversenodeID, 7, NTIM, vSCALE_N);
				}
			}
		}
		// printout process for remained parts later
		for (int kPAT = 0; kPAT < NPAT; kPAT++)
		{
			dumSTRING = *partNAME[kPAT];
			if (dumSTRING.empty()) continue;
			if ((PartArray[kPAT]->getSECTION()) == 1) continue;

			if (x3dOption == 1) // WRL
			{
				// printout PartObject-case with caseNUMBER=1,2,3,4]
				PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 1, NTIM, vSCALE_N);
				PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 2, NTIM, vSCALE_N);
				PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 3, NTIM, vSCALE_N);
				PartArray[kPAT]->printVRML(outVRF, undeformed, animationTIME, inversenodeID, 4, NTIM, vSCALE_N);
				// read coordinate at each time step
				for (int iTIM = 0; iTIM < NTIM+1; iTIM++)
				{
					for (int iNOD = 0; iNOD < NNOD_temp; iNOD++)
						for (int j = 0; j < 3; j++)
							coordinate[iNOD][j] = history_data[iTIM][iNOD][j];
					// printout PartObject-case with caseNUMBER=5]
					PartArray[kPAT]->printVRML(outVRF, coordinate, animationTIME, inversenodeID, 5, NTIM, vSCALE_N);
				}
				// printout PartObject-case with caseNUMBER=6,7]
				PartArray[kPAT]->printVRML(outVRF, coordinate, animationTIME, inversenodeID, 6, NTIM, vSCALE_N);
				PartArray[kPAT]->printVRML(outVRF, coordinate, animationTIME, inversenodeID, 7, NTIM, vSCALE_N);
			}
			else if (x3dOption == 2)
			{
				// printout PartObject-case with caseNUMBER=1,2,3,4]
				PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 1, NTIM, vSCALE_N);
				PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 2, NTIM, vSCALE_N);
				PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 3, NTIM, vSCALE_N);
				PartArray[kPAT]->printXML(outVRF, undeformed, animationTIME, inversenodeID, 4, NTIM, vSCALE_N);
				// read coordinate at each time step
				for (int iTIM = 0; iTIM < NTIM+1; iTIM++)
				{
					for (int iNOD = 0; iNOD < NNOD_temp; iNOD++)
						for (int j = 0; j < 3; j++)
							coordinate[iNOD][j] = history_data[iTIM][iNOD][j];
					// printout PartObject-case with caseNUMBER=5]
					PartArray[kPAT]->printXML(outVRF, coordinate, animationTIME, inversenodeID, 5, NTIM, vSCALE_N);
				}
				// printout PartObject-case with caseNUMBER=6,7]
				PartArray[kPAT]->printXML(outVRF, coordinate, animationTIME, inversenodeID, 6, NTIM, vSCALE_N);
				PartArray[kPAT]->printXML(outVRF, coordinate, animationTIME, inversenodeID, 7, NTIM, vSCALE_N);
			}
		}

		// printout part[1] of Overall-case with caseNUMBER=2]
		if (x3dOption == 1) printWRL(outVRF, partNAME, NPAT, NPAT_temp, 2, animationTIME[NTIM], aOption, vSCALE_N, dimension, flagDELETE, timeDELETE, animationTIME[NTIM]);
		if (x3dOption == 2) printX3D(outVRF, partNAME, NPAT, NPAT_temp, 2, animationTIME[NTIM], aOption, vSCALE_N, dimension, fileNAME);

		outVRF.close();
	}
	//*********************************************************
	// [end of aOption = 2 : dynamic static VRML (Simulation) ]

	// file open for html file to combine VRML browser and Java Applet together
	fileTAG = ".htm";
	fileNAME = outputNAME + fileTAG;
	outHTM.open(fileNAME.c_str());

	outHTM << "<HTML>" << endl;
	outHTM << "  <HEAD>" << endl;
	outHTM << "    <TITLE> FEM Visualization based on VRML and the EAI </TITLE>" << endl;
	outHTM << "  </HEAD>" << endl;
	outHTM << "   <BODY>" << endl;
	outHTM << "    <!-- VRML plugged-in scene -->" << endl;
	outHTM << "    <embed src=\"" << outputNAME << ".wrl" << "\" border=0 height=\"620\" width=\"1000\"" << endl;
	outHTM << "    PLUGINSPAGE=\"http://www.parallelgraphics.com/cortona\">" << endl;
	outHTM << "    <!-- EAI Java applet -->" << endl;
	outHTM << "    <applet code=\"FemVrmlJApplet.class\" mayscript  height=\"620\" width=155\">" << endl;
	outHTM << "    </applet>" << endl;
	outHTM << "  </BODY>" << endl;
	outHTM << "</HTML>" << endl;

	outHTM.close();

//===================================================================//
//                                                                   //
//       PROCESS <4> CLOSE the PROGRAM                               //
//                                                                   //
//===================================================================//

	delete[] nodeID;
	delete[] inversenodeID;

	for (int i = 0; i < NNOD_temp; i++)
	{
		delete[] undeformed[i];
		delete[] coordinate[i];
	}
	delete[] undeformed;
	delete[] coordinate;

	for (int i = 0; i < (NTIM+1); i++)
		for (int j = 0; j < NNOD_temp; j++)
			delete[] history_data[i][j];

	for (int i = 0; i < NELE; i++)
	{
		delete[] element[i];
		delete[] e_surfx[i];
	}
	delete[] element;
	delete[] e_surfx;

	delete[] tempNODE;
	delete[] tempSURFACE;
	
	delete[] spacingTIME;
	delete[] animationTIME;

	delete[] partNAME;
	delete[] flagDELETE;
	delete[] timeDELETE;

	delete[] PartArray;
	delete[] SectionArray;

	return;
}


//*************************************************************************
// getDIMEN calculate maximum length of each x,y,z direction roughly
//*************************************************************************
void getDIMEN(DoubleArrayPtr* coordinate, double dimension[][3], int NNOD_temp)
{
	double x_max, x_min, y_max, y_min, z_max, z_min;

	for (int i = 0; i < NNOD_temp; i++)
	{
		if (i == 0) 
		{
			x_max = coordinate[i][0];
			x_min = coordinate[i][0];
			y_max = coordinate[i][1];
			y_min = coordinate[i][1];
			z_max = coordinate[i][2];
			z_min = coordinate[i][2];
		}
		else
		{
			if (coordinate[i][0] > x_max) x_max = coordinate[i][0];
			if (coordinate[i][0] < x_min) x_min = coordinate[i][0];
			if (coordinate[i][1] > y_max) y_max = coordinate[i][1];
			if (coordinate[i][1] < y_min) y_min = coordinate[i][1];
			if (coordinate[i][2] > z_max) z_max = coordinate[i][2];
			if (coordinate[i][2] < z_min) z_min = coordinate[i][2];
		}
	}
	// dimension[0][0~2] : maximum length of each direction
	// dimension[1][0~2] : center position
	dimension[0][0] = x_max - x_min;
	dimension[0][1] = y_max - y_min;
	dimension[0][2] = z_max - z_min;
	dimension[1][0] = (x_max + x_min) / 2;
	dimension[1][1] = (y_max + y_min) / 2;
	dimension[1][2] = (z_max + z_min) / 2;
}


//********************************************************************
// extractSURFX extract surface data for solid element and store 
//              element number for each surface(face) in e_surfx array
//********************************************************************
void extractSURFX(IntArrayPtr* element,IntArrayPtr* e_surfx, int NELE)
{
	int	i_NOD[8],j_NOD[8],ij_NS[4];

	// e_surfx[NELE][6] : element number for each surface(face) of solid element
	//		= 0			- there is no neighbour element for this surface
	//		= positive	- there is a neighbour element for this surface, in same part
	//		= negative	- there is a neighbour element for this surface, not in same part
	//		= 9999999	- not existing surface due to wedge element

    for (int iELE = 0; iELE < NELE; iELE++) 
	{
		if (element[iELE][0] == 0) continue; // not available element (blank for this program)
		if (element[iELE][0] == 2) continue; // beam element
		if (element[iELE][0] == 4) continue; // shell element
		if (element[iELE][0] == 6)	e_surfx[iELE][5] = 9999999; // wedge element (6-node solid element)

		for (int i = 0; i < 8; i++)	i_NOD[i] = element[iELE][2+i];
		for (int jELE = 0; jELE < NELE; jELE++)
		{
			if (iELE == jELE) continue;

            int count = 0;
			for (int j = 0; j < 8; j++)	j_NOD[j] = element[jELE][2+j];
			for (int ins = 0; ins < 8; ins++)
			{
                if ((element[iELE][0] == 6) && ((ins == 5) || (ins == 7))) continue;
				for (int jns = 0; jns < 8; jns++)
				{
					if ((element[jELE][0] == 6) && ((jns == 5) || (jns == 7))) continue;
					// element iELE shares the same node with element jELE
					if (i_NOD[ins] == j_NOD[jns])
					{
						ij_NS[count] = i_NOD[ins];
						count++;
						break;
					}
				}
			}


			// "number of shared nodes is 4" means that these two elements share the same face
			if ((count == 4) || ((count == 3) && (element[iELE][0] == 6)))
			{
				// f_ins : face index for element iELE & f_jns : face index for element jELE
				int f_ins = -999, f_jns = -999;
				for (int iSUF = 0; iSUF < 6; iSUF++)
				{
                    int k_ins = 0, k_jns = 0;
					for (int i = 0; i < 4; i++)
					{
						for (int j = 0; j < 4; j++)
						{
                            if (ij_NS[i] == i_NOD[NNofFACE[iSUF][j]-1]) k_ins++;
                            if (ij_NS[i] == j_NOD[NNofFACE[iSUF][j]-1]) k_jns++;
                        }
					}
					// find the face number for each element iELE, jELE
					if ((k_ins >= 4) && !((iSUF == 5 ) && (element[iELE][0] == 6))) f_ins = iSUF;
					if ((k_jns >= 4) && !((iSUF == 5 ) && (element[jELE][0] == 6))) f_jns = iSUF;
				}
				if ((f_ins == -999) || (f_jns == -999)) continue;

				// element iELE, jELE are not in same part
				if (element[iELE][1] != element[jELE][1])
				{
					e_surfx[iELE][f_ins] = -(jELE+1);
					e_surfx[jELE][f_jns] = -(iELE+1);
				}
				// element iELE, jELE are in same part
				else
				{
					e_surfx[iELE][f_ins] = jELE+1;
					e_surfx[jELE][f_jns] = iELE+1;
				}
			}
		}
	}
}


//************************************************************************
// getSIZE : calculate the number of Surface(size_SF) 
//           and NoDe index(size_ND) for each part
//************************************************************************
void getSIZE(IntArrayPtr* element, IntArrayPtr* e_surfx, 
			  IntArrayPtr tempNODE, IntArrayPtr tempSURFACE,
			  int NELE, int kPAT, int& kTYPE, int& size_SF, int& size_ND)
{
	// i_number_SF : total number of surfaces & i_number_ND : total number of nodes
	// repeated : flag to check the repeated node index for storing tempNODE array
	int i_number_SF = 0, i_number_ND = 0;
	int repeated;

	for (int i = 0; i < NELE; i++)
	{
		if (element[i][0] == 0) continue; // not available element (blank for this program)

		// find element index for PartObject
		if (element[i][1] == kPAT) 
		{
			// (1) For solid element
			if ((element[i][0] == 6) || (element[i][0] == 8)) 
			{
				kTYPE = 1;
                // find surface index for PartObject
				for (int iSUF = 0; iSUF < 6; iSUF++)
				{
					if ((e_surfx[i][iSUF] <= 0) && ((element[i][0] != 6) || (iSUF != 5)))
					{
						for (int j = 0; j <4; j++)
						{
							// store node connectivity for PartObject
							tempSURFACE[4*(i_number_SF)+j]=element[i][2+NNofFACE[iSUF][j]-1];
							repeated = 0;
							for (int k = 0; k < i_number_ND; k++)
								if(tempSURFACE[4*(i_number_SF)+j] == tempNODE[k])
								{
									repeated = 1;
									break;
								}
							// store node id(Not repeated) for PartObject
							if (repeated != 1) 
							{
								tempNODE[i_number_ND] = tempSURFACE[4*(i_number_SF)+j];
								i_number_ND++;
							}
						}
						i_number_SF++;
					}
				}
			}
			// (2) For plate/shell element
			else if (element[i][0] == 4)
			{
				kTYPE = 2;
				for (int j = 0; j <4; j++)
				{
					// store node connectivity for PartObject
					tempSURFACE[4*(i_number_SF)+j]=element[i][2+j];
					repeated = 0;
					for (int k = 0; k < i_number_ND; k++)
						if(tempSURFACE[4*(i_number_SF)+j] == tempNODE[k])
						{
							repeated = 1;
							break;
						}
					// store node id(Not repeated) for PartObject
					if (repeated != 1) 
					{
						tempNODE[i_number_ND] = tempSURFACE[4*(i_number_SF)+j];
						i_number_ND++;
					}
				}
				i_number_SF++;
			}
			// (3) For beam element
			else if (element[i][0] == 2)
			{
				kTYPE = 3;
				for (int j = 0; j <4; j++)
				{
					// store node connectivity for PartObject
					if (j < 2) tempSURFACE[4*(i_number_SF)+j]=element[i][2+j];
					if (j > 1) tempSURFACE[4*(i_number_SF)+j]=element[i][2+3-j];
					repeated = 0;
					for (int k = 0; k < i_number_ND; k++)
						if(tempSURFACE[4*(i_number_SF)+j] == tempNODE[k])
						{
							repeated = 1;
							break;
						}
					// store node id(Not repeated) for PartObject
						if (repeated != 1) 
						{
							tempNODE[i_number_ND] = tempSURFACE[4*(i_number_SF)+j];
							i_number_ND++;
						}
				}
				i_number_SF++;
			}
		}
	}
	size_SF = i_number_SF;
	size_ND = i_number_ND;
}


//***********************************************************************************
// printWRL printout Overall-case data in *.WRL
//***********************************************************************************
void printWRL(ofstream& outWRL, StringArrayPtr *partNAME, 
			  int NPAT, int NPAT_temp, int caseNUMBER, double cycleINTERVAL, int aOption,
			  double vSCALE_N, double dimension[][3],
			  IntArrayPtr flagDELETE, DoubleArrayPtr timeDELETE, double endTIME)
{
	string dumSTRING;

	//******************
	if (caseNUMBER == 1)
	//******************
	{
		double maxLength = pow ((dimension[0][0] * dimension[0][1] * dimension[0][2]), (1/3.));
		double avatarSize = 0.03 * maxLength * vSCALE_N;

		// # Description
		outWRL << "#VRML V2.0 utf8" << endl << endl;

		outWRL << "DEF AVATAR NavigationInfo {" << endl;
		outWRL << "   avatarSize [ " << avatarSize << " , " << avatarSize << " , " << avatarSize << " ]" << endl;
		outWRL << "}" << endl;

		//# ViewPoint Setting #
		outWRL << "Group {" << endl;
		outWRL << " children [" << endl;
		//  normal viewpoint
		outWRL << "   DEF Normal Viewpoint {" << endl;
		outWRL << "    position " << dimension[1][0]*vSCALE_N << " " << (dimension[1][1] - dimension[0][1]/2 - maxLength)*vSCALE_N << " " << dimension[1][2]*vSCALE_N << endl;
		outWRL << "    orientation 0.99999999 0 0 1.55079899" << endl;
		outWRL << "    fieldOfView 0.78" << endl;
		outWRL << "    description \"Normal View\"" << endl;
		outWRL << "   }" << endl;
		//  front viewpoint
		outWRL << "   DEF Front Viewpoint {" << endl;
		outWRL << "    position " << (dimension[1][0] + dimension[0][0]/2 + maxLength)*vSCALE_N << " " << dimension[1][1]*vSCALE_N << " " << dimension[1][2]*vSCALE_N << endl;
		outWRL << "    orientation 0.57735026 0.57735026 0.57735026 2.09439510" << endl;
		outWRL << "    fieldOfView 0.78" << endl;
		outWRL << "    description \"Front View\"" << endl;
		outWRL << "   }" << endl;
		//  rear viewpoint
		outWRL << "   DEF Rear Viewpoint {" << endl;
		outWRL << "    position " << (dimension[1][0] - dimension[0][0]/2 - maxLength)*vSCALE_N << " " << dimension[1][1]*vSCALE_N << " " << dimension[1][2]*vSCALE_N << endl;
		outWRL << "    orientation 0.57735026 -0.57735026 -0.57735026 2.09439510" << endl;
		outWRL << "    fieldOfView 0.78" << endl;
		outWRL << "    description \"Rear View\"" << endl;
		outWRL << "   }" << endl;
		//  top viewpoint
		outWRL << "   DEF Top Viewpoint {" << endl;
		outWRL << "    position " << dimension[1][0]*vSCALE_N << " " << dimension[1][1]*vSCALE_N << " " << (dimension[1][2] + dimension[0][2]/2 + maxLength)*vSCALE_N << endl;
		outWRL << "    orientation 1 0 0 0 " << endl;
		outWRL << "    fieldOfView 0.78" << endl;
		outWRL << "    description \"Top View\"" << endl;
		outWRL << "   }" << endl;
		//  bottom viewpoint
		outWRL << "   DEF Bottom Viewpoint {" << endl;
		outWRL << "    position " << dimension[1][0]*vSCALE_N << " " << dimension[1][1]*vSCALE_N << " " << (dimension[1][2] - dimension[0][2]/2 - maxLength)*vSCALE_N << endl;
		outWRL << "    orientation 1 0 0 3.14159265" << endl;
		outWRL << "    fieldOfView 0.78" << endl;
		outWRL << "    description \"Bottom View\"" << endl;
		outWRL << "   }" << endl;
		//  right-side viewpoint
		outWRL << "   DEF Right_Side Viewpoint {" << endl;
		outWRL << "    position " << dimension[1][0]*vSCALE_N << " " << (dimension[1][1] + dimension[0][1]/2 + maxLength)*vSCALE_N << " " << dimension[1][2]*vSCALE_N << endl;
		outWRL << "    orientation -4.28613098e-17 -0.70000142 -0.714141442 3.14159265" << endl;
		outWRL << "    fieldOfView 0.78" << endl;
		outWRL << "    description \"Right View\"" << endl;
		outWRL << "   }" << endl;
		//  left-side viewpoint
		outWRL << "   DEF Left_Side Viewpoint {" << endl;
		outWRL << "    position " << dimension[1][0]*vSCALE_N << " " << (dimension[1][1] - dimension[0][1]/2 - maxLength)*vSCALE_N << " " << dimension[1][2]*vSCALE_N << endl;
		outWRL << "    orientation 0.99999999 0 0 1.55079899" << endl;
		outWRL << "    fieldOfView 0.78" << endl;
		outWRL << "    description \"Left View\"" << endl;
		outWRL << "   }" << endl;
		outWRL << "  ]" << endl;
		outWRL << "}" << endl << endl;

		//# TimeSensor Node #
		outWRL << "DEF Clock TimeSensor {" << endl;
		outWRL << "	cycleInterval " << cycleINTERVAL << endl;
		outWRL << "}" << endl << endl;

		//# ClockControl Script Node #
		outWRL << "DEF ClockControl Script {" << endl;
		outWRL << "	eventIn SFFloat set_fraction" << endl;
		outWRL << "	eventIn SFBool IsReverse" << endl;
		outWRL << "	eventOut SFFloat fraction_changed" << endl;
		outWRL << "	url \"javascript:" << endl;
		outWRL << "	function init() {" << endl;
		outWRL << "		IsReverse = FALSE;" << endl;
		outWRL << "	}" << endl;
		outWRL << "	function set_fraction(value) {" << endl;
		outWRL << "		if (IsReverse == TRUE) fraction_changed = 1.0 - value;" << endl;
		outWRL << "		else fraction_changed = value;" << endl;
		outWRL << "	}" << endl;
		outWRL << "	\"" << endl;
		outWRL << "}" << endl << endl;

		//# ViewPartName Script Node #
		outWRL << "DEF ViewPartName Script {" << endl;
		outWRL << "	eventIn SFString partName" << endl;
		outWRL << "	eventOut SFString PartNameText" << endl;
		outWRL << "	url \"javascript:" << endl;
		outWRL << "	function init() {" << endl;
		outWRL << "		PartNameText = '';" << endl;
		outWRL << "	}" << endl;
		outWRL << "	function partName(value) {" << endl;
		outWRL << "		PartNameText = value;" << endl;
		outWRL << "	}" << endl;
		outWRL << "	\"" << endl;
		outWRL << "}" << endl << endl;

		//# TouchControl Script Node #
		outWRL << "DEF TouchControl Script {" << endl;
		outWRL << "	eventIn SFBool TouchEnabled" << endl;
		outWRL << "	eventOut SFBool enabled" << endl;
		outWRL << "	url \"javascript:" << endl;
		outWRL << "	function init() {" << endl;
		outWRL << "		enabled = TRUE;" << endl;
		outWRL << "	}" << endl;
		outWRL << "	function TouchEnabled(value) {" << endl;
		outWRL << "		enabled = value;" << endl;
		outWRL << "	}" << endl;
		outWRL << "	\"" << endl;
		outWRL << "}" << endl << endl;

		//# InputData Script Node # -- Dummy Script to give the names of each part object to Java Applet
		outWRL << "DEF InputData Script {" << endl;
	    outWRL << "	field SFInt32 numberOfParts 0" << endl;
		outWRL << "	field MFString partName []" << endl;
	    outWRL << "	url \"javascript:" << endl;
		outWRL << "	function initialize() {" << endl;
	    outWRL << "		numberOfParts = " << NPAT_temp <<";" << endl;
		int iPAT = 0;
		for (int i = 0; i < NPAT; i++)
		{
			dumSTRING = *partNAME[i];
			if (dumSTRING.empty()) continue;
			outWRL << "		partName[" << iPAT << "] = \'" << *partNAME[i] << "\';" << endl;
			iPAT++;
		}
		outWRL << "	}" << endl;
		outWRL << "	\"" << endl;
		outWRL << "}" << endl << endl;
	
	} // end of caseNUMBER = 1


	//************************
	else if (caseNUMBER == 2)
	//************************
	{
		//# TextVisitble Script Node #
		outWRL << endl << endl;

		outWRL << "DEF AllTextVisibleScript Group {" << endl;
		outWRL << "  children [" << endl << endl;
		for (int i = 0; i < NPAT; i++)
		{
			dumSTRING = *partNAME[i];
			if (dumSTRING.empty()) continue;

			outWRL << "DEF " << *partNAME[i] << "_TextVisible Script {" << endl;
			outWRL << "	eventIn SFBool isOver" << endl;
			outWRL << "	eventOut SFString partName" << endl;
			outWRL << "	url \"javascript:" << endl;
			outWRL << "	function initialize() {" << endl;
			outWRL << "		partName = '';" << endl;
			outWRL << "	}" << endl;
			outWRL << "	function isOver(value, time) {" << endl;
			outWRL << "		if (value == TRUE) partName = '" << *partNAME[i] << "';" << endl;
			outWRL << "		else partName = '';" << endl;
			outWRL << "	}" << endl;
			outWRL << "	\"" << endl;
			outWRL << "}" << endl << endl;
		}
		outWRL << endl;
		outWRL << "  ]" << endl;
		outWRL <<"}" << endl << endl;

		//# BodyVisiebl Script Node #
		outWRL << endl << endl;

		outWRL << "DEF AllBodyVisibleScript Group {" << endl;
		outWRL << "  children [" << endl << endl;
		for (int i = 0; i < NPAT; i++)
		{
			dumSTRING = *partNAME[i];
			if (dumSTRING.empty()) continue;

			outWRL << "DEF " << *partNAME[i] << "_BodyVisible Script {" << endl;
			outWRL << "	eventIn SFTime touchTime" << endl;
			outWRL << "	field SFInt32 number 2" << endl;
			outWRL << "	eventOut SFInt32 output" << endl;
			outWRL << "	url \"javascript:" << endl;
			outWRL << "	function initialize() {" << endl;
			outWRL << "		output = 0;" << endl;
			outWRL << "	}" << endl;
			outWRL << "	function touchTime(value, time) {" << endl;
			outWRL << "		if (output == number - 1) output = 0;" << endl;
			outWRL << "		else ++output;" << endl;
			outWRL << "	}" << endl;
			outWRL << "	\"" << endl;
			outWRL << "}" << endl;
		}
		outWRL << endl;
		outWRL << "  ]" << endl;
		outWRL <<"}" << endl << endl;

		//# DELETE Script Node #
		outWRL << endl << endl;

		outWRL << "DEF AllDeleteScript Group {" << endl;
		outWRL << "  children [" << endl << endl;
		for (int i = 0; i < NPAT; i++)
		{
			dumSTRING = *partNAME[i];
			if (dumSTRING.empty()) continue;

			if (flagDELETE[i] == 1)
			{
				outWRL << "DEF " << *partNAME[i] << "_DELETE Script {" << endl;
				outWRL << "	eventIn SFFloat set_fraction" << endl;
				outWRL << "	eventOut SFInt32 output" << endl;
				outWRL << "	field SFFloat deleteFraction " << timeDELETE[i]/endTIME << endl;
				outWRL << "	url \"javascript:" << endl;
				outWRL << "	function initialize() {" << endl;
				outWRL << "		output = 0;" << endl;
				outWRL << "	}" << endl;
				outWRL << "	function set_fraction(value, time) {" << endl;
				outWRL << "		if (value <= deleteFraction) output = 0;" << endl;
				outWRL << "		if (value > deleteFraction) output = 1;" << endl;
				outWRL << "	}" << endl;
				outWRL << "	\"" << endl;
				outWRL << "}" << endl;
			}
		}
		outWRL << endl;
		outWRL << "  ]" << endl;
		outWRL <<"}" << endl << endl;

		//# ROUTE Setting #
		outWRL <<"ROUTE Clock.fraction_changed TO ClockControl.set_fraction" << endl << endl;
		for (int i = 0; i < NPAT; i++)
		{
			dumSTRING = *partNAME[i];
			if (dumSTRING.empty()) continue;

			if (aOption == 2)
			{
				outWRL << "ROUTE ClockControl.fraction_changed TO " << *partNAME[i] << "_Morph.set_fraction" << endl;
				outWRL << "ROUTE " << *partNAME[i] << "_Morph.value_changed TO " << *partNAME[i] << "_Coordinate.set_point" << endl;

				if (flagDELETE[i] == 1) 
				{
					outWRL << "ROUTE ClockControl.fraction_changed TO " << *partNAME[i] << "_DELETE.set_fraction" << endl;
					outWRL << "ROUTE " << *partNAME[i] << "_DELETE.output TO " << *partNAME[i] << "_BodySwitch.whichChoice" << endl;
				}
			}
			outWRL << "ROUTE " << *partNAME[i] << "_BodyTouch.isOver TO " << *partNAME[i] << "_TextVisible.isOver" << endl;
			outWRL << "ROUTE " << *partNAME[i] << "_TextVisible.partName TO " << "ViewPartName.partName" << endl;
			outWRL << "ROUTE TouchControl.enabled TO " << *partNAME[i] << "_BodyTouch.set_enabled" << endl;
			outWRL << "ROUTE " << *partNAME[i] << "_BodyTouch.touchTime TO " << *partNAME[i] << "_BodyVisible.touchTime" << endl;
			outWRL << "ROUTE " << *partNAME[i] << "_BodyVisible.output TO " << *partNAME[i] << "_BodySwitch.whichChoice" << endl << endl;
		}
	} // end of caseNUMBER = 2
}


//***************************************************************************************
// printX3D printout Overall-case data in *.X3D
//***************************************************************************************
void printX3D(ofstream& outWRL, StringArrayPtr *partNAME, 
			  int NPAT, int NPAT_temp, int caseNUMBER, double cycleINTERVAL, int aOption,
			  double vSCALE_N, double dimension[][3], string& fileNAME)
{
	string dumSTRING;

	//******************
	if (caseNUMBER == 1) 
	//******************
	{
		// # Description
		outWRL << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
		outWRL << "<!DOCTYPE X3D PUBLIC ""ISO//Web3D//DTD X3D 3.0//EN"" ""http://www.web3d.org/specifications/x3d-3.0.dtd"">" << endl;
		outWRL << "<X3D profile=\"Full\">" << endl;
		outWRL << "  <head>" << endl;
		outWRL << "    <meta name=\"filename\" content=\"" << fileNAME << ".x3d\"/>" << endl;
		outWRL << "  </head>" << endl;
		outWRL << "  <Scene>" << endl;
        	
		//# ViewPoint Setting #
		double maxLength = pow ((dimension[0][0] * dimension[0][1] * dimension[0][2]), (1/3.));
        	
		outWRL << "  <Group >" << endl;
		//  normal viewpoint
		outWRL << "      <Viewpoint  DEF=\"Normal\" " 
			   << "position=\"" << dimension[1][0]*vSCALE_N << " " << (dimension[1][1] - dimension[0][1]/2 - maxLength)*vSCALE_N << " " << dimension[1][2]*vSCALE_N << "\" "
			   << "orientation=\"0.99999999 0 0 1.55079899\" "
			   << "description=\"Normal View\" fieldOfView=\"0.78\"/>" << endl;
		//  front viewpoint
		outWRL << "      <Viewpoint  DEF=\"Front\" "
			   << "position=\"" << (dimension[1][0] + dimension[0][0]/2 + maxLength)*vSCALE_N << " " << dimension[1][1]*vSCALE_N << " " << dimension[1][2]*vSCALE_N << "\" "
			   << "orientation=\"0.57735026 0.57735026 0.57735026  2.0943952\" "
			   << "description=\"Front View\" fieldOfView=\"0.78\"/>" << endl;
		//  rear viewpoint
		outWRL << "      <Viewpoint  DEF=\"Rear\" " 
			   << "position=\"" << (dimension[1][0] - dimension[0][0]/2 - maxLength)*vSCALE_N << " " << dimension[1][1]*vSCALE_N << " " << dimension[1][2]*vSCALE_N << "\" "
			   << "orientation=\"0.57735026 -0.57735026 -0.57735026 2.09439510\" "
			   << "description=\"Rear View\" fieldOfView=\"0.78\"/>" << endl;
		//  top viewpoint
		outWRL << "      <Viewpoint  DEF=\"Top\" " 
			   << "position=\"" << dimension[1][0]*vSCALE_N << " " << dimension[1][1]*vSCALE_N << " " << (dimension[1][2] + dimension[0][2]/2 + maxLength)*vSCALE_N << "\" "
			   << "orientation=\"1.0 0.0 0.0  0.0\" "
			   << "description=\"Top View\" fieldOfView=\"0.78\"/>" << endl;
		//  bottom viewpoint
		outWRL << "      <Viewpoint  DEF=\"Bottom\" " 
			   << "position=\"" << dimension[1][0]*vSCALE_N << " " << dimension[1][1]*vSCALE_N << " " << (dimension[1][2] - dimension[0][2]/2 - maxLength)*vSCALE_N << "\" "
			   << "orientation=\"1.0 0.0 0.0  3.1415927\" "
			   << "description=\"Bottom View\" fieldOfView=\"0.78\"/>" << endl;
		//  right-side viewpoint
		outWRL << "      <Viewpoint  DEF=\"Right_Side\" " 
			   << "position=\"" << dimension[1][0]*vSCALE_N << " " << (dimension[1][1] + dimension[0][1]/2 + maxLength)*vSCALE_N << " " << dimension[1][2]*vSCALE_N << "\" "
			   << "orientation=\"-4.286131E-17 -0.7000014 -0.7141414  3.1415927\" "
			   << "description=\"Right View\" fieldOfView=\"0.78\"/>" << endl;
		//  left-side viewpoint
		outWRL << "      <Viewpoint  DEF=\"Left_Side\" " 
			   << "position=\"" << dimension[1][0]*vSCALE_N << " " << (dimension[1][1] - dimension[0][1]/2 - maxLength)*vSCALE_N << " " << dimension[1][2]*vSCALE_N << "\" "
			   << "orientation=\"1.0 0.0 0.0  1.550799\" "
			   << "description=\"Left View\" fieldOfView=\"0.78\"/>" << endl;
		outWRL << "  </Group>" << endl << endl;
        	
        	
		//# TimeSensor Node #
		outWRL << "  <TimeSensor  DEF=\"Clock\"  cycleInterval=\"" << cycleINTERVAL <<"\"/>" << endl << endl;
        	
		//# ClockControl Script Node #
		outWRL << "  <Script DEF=\"ClockControl\">" << endl;
		outWRL << "    <field name=\"fraction_changed\" type=\"SFFloat\" accessType=\"outputOnly\"/>" << endl;
		outWRL << "    <field name=\"IsReverse\" type=\"SFBool\" accessType=\"inputOnly\"/>" << endl;
		outWRL << "    <field name=\"set_fraction\" type=\"SFFloat\" accessType=\"inputOnly\"/>" << endl;
		outWRL << "    <![CDATA[javascript:" << endl;
		outWRL << "	function init() {" << endl;
		outWRL << "		IsReverse = FALSE;" << endl;
		outWRL << "	}" << endl;
		outWRL << "	function set_fraction(value) {" << endl;
		outWRL << "		if (IsReverse == TRUE) fraction_changed = 1.0 - value;" << endl;
		outWRL << "		else fraction_changed = value;" << endl;
		outWRL << "	}" << endl;
		outWRL << "    ]]>" << endl;
		outWRL << "  </Script>" << endl << endl;
        	
		//# ViewPartName Script Node #
		outWRL << "  <Script DEF=\"ViewPartName\">" << endl;
		outWRL << "    <field name=\"partName\" type=\"SFString\" accessType=\"inputOnly\"/>" << endl;
		outWRL << "    <field name=\"PartNameText\" type=\"SFString\" accessType=\"outputOnly\"/>" << endl;
		outWRL << "    <![CDATA[javascript:" << endl;
		outWRL << "	function init() {" << endl;
		outWRL << "		PartNameText = '';" << endl;
		outWRL << "	}" << endl;
		outWRL << "	function partName(value) {" << endl;
		outWRL << "		PartNameText = value;" << endl;
		outWRL << "	}" << endl;
		outWRL << "    ]]>" << endl;
		outWRL << "  </Script>" << endl << endl;
        	
		//# TouchControl Script Node #
		outWRL << "  <Script DEF=\"TouchControl\">" << endl;
		outWRL << "    <field name=\"TouchEnabled\" type=\"SFBool\" accessType=\"inputOnly\"/>" << endl;
		outWRL << "    <field name=\"enabled\" type=\"SFBool\" accessType=\"outputOnly\"/>" << endl;
		outWRL << "    <![CDATA[javascript:" << endl;
		outWRL << "	function init() {" << endl;
		outWRL << "		enabled = TRUE;" << endl;
		outWRL << "	}" << endl;
		outWRL << "	function TouchEnabled(value) {" << endl;
		outWRL << "		enabled = value;" << endl;
		outWRL << "	}" << endl;
		outWRL << "    ]]>" << endl;
		outWRL << "  </Script>" << endl << endl;
        	
		//# InputData Script Node # -- Dummy Script to give the names of each part object to Java Applet
		outWRL << "  <Script DEF=\"InputData\">" << endl;
		outWRL << "    <field name=\"partName\" type=\"MFString\" value='' accessType=\"initializeOnly\"/>" << endl;
		outWRL << "    <field name=\"numberOfParts\" type=\"SFInt32\" value=\"0\" accessType=\"initializeOnly\"/>" << endl;
		outWRL << "    <![CDATA[javascript:" << endl;
		outWRL << "	function initialize() {" << endl;
		outWRL << "		numberOfParts = " << NPAT_temp <<";" << endl;
		int iPAT = 0;
		for (int i = 0; i < NPAT; i++)
		{
			dumSTRING = *partNAME[i];
			if (dumSTRING.empty()) continue;
			outWRL << "		partName[" << iPAT << "] = \'" << *partNAME[i] << "\';" << endl;
			iPAT++;
		}
		outWRL << "	}" << endl;
		outWRL << "    ]]>" << endl;
		outWRL << "  </Script>" << endl << endl;
	} // end of caseNUMBER = 1

	//***********************
	else if (caseNUMBER == 2)
	//***********************
	{
		//# TextVisitble Script Node #
		outWRL << endl << endl;
        	
		outWRL << "  <Group  DEF=\"AllTextVisibleScript\" >" << endl;
		for (int i = 0; i < NPAT; i++)
		{
			dumSTRING = *partNAME[i];
			if (dumSTRING.empty()) continue;
        	
			outWRL << "      <Script DEF=\"" << *partNAME[i] << "_TextVisible\">" << endl;
			outWRL << "        <field name=\"partName\" type=\"SFString\" accessType=\"outputOnly\"/>" << endl;
			outWRL << "        <field name=\"isOver\" type=\"SFBool\" accessType=\"inputOnly\"/>" << endl;
			outWRL << "        <![CDATA[javascript:" << endl;
			outWRL << "	function initialize() {" << endl;
			outWRL << "		partName = '';" << endl;
			outWRL << "	}" << endl;
			outWRL << "	function isOver(value, time) {" << endl;
			outWRL << "		if (value == TRUE) partName = '" << *partNAME[i] << "';" << endl;
			outWRL << "		else partName = '';" << endl;
			outWRL << "	}" << endl;
			outWRL << "        ]]>" << endl;
			outWRL << "      </Script>" << endl << endl;
		}
		outWRL << "  </Group>" << endl;
        	
		//# BodyVisiebl Script Node #
		outWRL << endl << endl;
        	
		outWRL << "  <Group  DEF=\"AllBodyVisibleScript\" >" << endl;
		for (int i = 0; i < NPAT; i++)
		{
			dumSTRING = *partNAME[i];
			if (dumSTRING.empty()) continue;
        	
			outWRL << "      <Script DEF=\"" << *partNAME[i] << "_BodyVisible\">" << endl;
			outWRL << "        <field name=\"output\" type=\"SFInt32\" accessType=\"outputOnly\"/>" << endl;
			outWRL << "        <field name=\"touchTime\" type=\"SFTime\" accessType=\"inputOnly\"/>" << endl;
			outWRL << "        <field name=\"number\" type=\"SFInt32\" value=\"2\" accessType=\"initializeOnly\"/>" << endl;
			outWRL << "        <![CDATA[javascript:" << endl;
			outWRL << "	function initialize() {" << endl;
			outWRL << "		output = 0;" << endl;
			outWRL << "	}" << endl;
			outWRL << "	function touchTime(value, time) {" << endl;
			outWRL << "		if (output == number - 1) output = 0;" << endl;
			outWRL << "		else ++output;" << endl;
			outWRL << "	}" << endl;
			outWRL << "        ]]>" << endl;
			outWRL << "      </Script>" << endl;
		}
		outWRL << "  </Group>" << endl;
        	
		//# ROUTE Setting #
		outWRL << endl << endl;
        	
		outWRL <<"<ROUTE fromNode=\"Clock\" fromField=\"fraction_changed\" toNode=\"ClockControl\" toField=\"set_fraction\"/>" << endl << endl;
		for (int i = 0; i < NPAT; i++)
		{
			dumSTRING = *partNAME[i];
			if (dumSTRING.empty()) continue;
        	
			if (aOption == 2)
			{
				outWRL << "<ROUTE fromNode=\"ClockControl\" fromField=\"fraction_changed\" toNode=\"" << *partNAME[i] << "_Morph\" toField=\"set_fraction\"/>" << endl;
				outWRL << "<ROUTE fromNode=\"" << *partNAME[i] << "_Morph\" fromField=\"value_changed\" toNode=\"" << *partNAME[i] << "_Coordinate\" toField=\"set_point\"/>" << endl;
			}
			outWRL << "<ROUTE fromNode=\"" << *partNAME[i] << "_BodyTouch\" fromField=\"isOver\" toNode=\"" << *partNAME[i] << "_TextVisible\" toField=\"isOver\"/>" << endl;
			outWRL << "<ROUTE fromNode=\"" << *partNAME[i] << "_TextVisible\" fromField=\"partName\" toNode=\"ViewPartName\" toField=\"partName\"/>" << endl;
			outWRL << "<ROUTE fromNode=\"TouchControl\" fromField=\"enabled\" toNode=\"" << *partNAME[i] << "_BodyTouch\" toField=\"set_enabled\"/>" << endl;
			outWRL << "<ROUTE fromNode=\"" << *partNAME[i] << "_BodyTouch\" fromField=\"touchTime\" toNode=\"" << *partNAME[i] << "_BodyVisible\" toField=\"touchTime\"/>" << endl;
			outWRL << "<ROUTE fromNode=\"" << *partNAME[i] << "_BodyVisible\" fromField=\"output\" toNode=\"" << *partNAME[i] << "_BodySwitch\" toField=\"whichChoice\"/>" << endl;
		}
		outWRL << "  </Scene>" << endl;
		outWRL << "</X3D>" << endl;
	} // end of caseNUMBER = 2

}


//**********************************************************************
// getHISTORY read displacement and calculate deformed shape 
//            and store it in history_data at each animationTIME step
//***********************************************************************
void getHISTORY(ifstream& inTME, ofstream& debug, IntArrayPtr inversenodeID,
			   DoubleArrayPtr* undeformed, DoubleArray2Ptr* history_data, 
			   DoubleArrayPtr animationTIME, IntArrayPtr spacingTIME,
			   int NNOD, int NNOD_temp, int NTIM, double vSCALE_D)
{
	string dumSTRING;
	char dumC;
	int nodeNUM;
	double xx, yy, zz;

	getline(inTME,dumSTRING);
	for (int i = 0; i < 14 ; i++) inTME.get(dumC);
	inTME >> animationTIME[0];
	getline(inTME,dumSTRING);

	int iTIM_initial = 0;

	// if the displacement history file start with non-zero time, make a undeformed coordinate as first step
	if (animationTIME[0] > 0.0)
	{
		animationTIME[1] = animationTIME[0];
		animationTIME[0] = 0.0;
		int iTIM = 0;
		iTIM_initial = 1;
		for (int i = 0; i < NNOD_temp; i++)
			for (int j = 0; j < 3; j++)
				history_data[iTIM][i][j] = undeformed[i][j];
	}

	for (int iTIM = iTIM_initial; iTIM < NTIM+1; iTIM++)
	{
		if (iTIM > iTIM_initial)
		{
			getline(inTME,dumSTRING);
			for (int i = 0; i < 14 ; i++) inTME.get(dumC);
			inTME >> animationTIME[iTIM];
			getline(inTME,dumSTRING);
		}
		getline(inTME,dumSTRING);
		getline(inTME,dumSTRING);

		for (int i = 0; i < NNOD; i++) 
		{
			inTME.get(dumC);
			if (dumC == '*') break;
			// read xx, yy, zz as displacement and calculate deformed coordinate with scale vSCALE_D 
			inTME >> nodeNUM >> xx >> yy >> zz;
			getline(inTME,dumSTRING);
			if ((inversenodeID[nodeNUM-1] < 0) || (inversenodeID[nodeNUM-1] >= NNOD_temp)) continue;
			history_data[iTIM][inversenodeID[nodeNUM-1]][0] = undeformed[inversenodeID[nodeNUM-1]][0] + (xx * vSCALE_D);
			history_data[iTIM][inversenodeID[nodeNUM-1]][1] = undeformed[inversenodeID[nodeNUM-1]][1] + (yy * vSCALE_D);
			history_data[iTIM][inversenodeID[nodeNUM-1]][2] = undeformed[inversenodeID[nodeNUM-1]][2] + (zz * vSCALE_D);
		}
		getline(inTME,dumSTRING);

		// skip according to time spacing data spacingTIME[iTIM]
		if ((iTIM < NTIM) && (spacingTIME[iTIM] > 1))
			for (int i = 0; i < (spacingTIME[iTIM]-1) * (4+NNOD_temp+1); i++) getline(inTME,dumSTRING);

	}

	debug << "in animationTIME:NTIM = " << NTIM << endl;
	for (int i = 0; i < (NTIM+1); i++)
	{
		debug << setw(10) << animationTIME[i];
		if ((i%10) == 9) debug << endl;
	}
	debug << endl;

}
